/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _AUDIO_SIGNAL_H_
#define _AUDIO_SIGNAL_H_

#include "Event.h"

class AudioSignal 
: public Object
{
	public:
		d_frame frame;

		int size;
		d_sample * data;

	public:
		AudioSignal (int s = 0, Type * t = AudioSignal_type)
			: Object (t)
			{ 
				frame = 0;
				size = s;
				data = 0;
				
				if (size)
					set_size (size);
			}

		AudioSignal (AudioSignal & e)
			: Object (AudioSignal_type)
			{
				data = 0;
				size = 0;
				set_size (e.size);
			
				*this = e;
			}

		~AudioSignal()
			{
				if (data) free (data);
			}

		void set_size (int n)
			{
				if (n == size && data)
					return;

				if (data) 
					free (data);

				size = n;
				data = (d_sample *) calloc (size, sizeof (d_sample));
	
				if (data == 0) 
					throw MemoryError() << "allocating a sample buffer of size " << n;
			}

		virtual void clone (AudioSignal * e) 
			{ 
				if (get_type (e) != get_type (this))
					throw TypeError() << "not the same type.";

				*this = *((AudioSignal *) e);
			}

		AudioSignal & operator = (AudioSignal & e)
			{
				if (e.size != size)
					throw AssertionError() << "unequally sized AudioSignal in assignment.";
				
				frame = e.frame;
				memcpy (data, e.data, size * sizeof (d_sample));

				return *this;
			}
	
		/* audio event methods */
		void silence()
			{
				for (int i = 0; i < size; i++)
					data[i] = (d_sample) 0;
			}
		
		AudioSignal & operator += (AudioSignal & e)
			{
				if (e.size != size)
					throw AssertionError() << "unequally sized (" 
							<< size << "," << e.size 
							<< ") AudioSignal in operator +.";
				
				d_sample * d = data, * s = e.data;
				int n = size;
				while (--n >= 0)
					*d += *s,
					++d, ++s;

				return *this;
			}
};

class CyclicAudio
: public AudioSignal
{
	public:
		CyclicAudio (int s = FRAMES_PER_CYCLE)
			: AudioSignal (s, CyclicAudio_type)
			{ }
};

#endif /* _AUDIO_SIGNAL_H_ */
