/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _BINARY_FILE_H_
#define _BINARY_FILE_H_

class BinaryFile
{
	public:
		char * path;
		int mode;

		int fd;

		BinaryFile()
			{
				fd = -1;
				path = 0;
			}

		~BinaryFile()
			{
				close();
			}

		void open (char * _path, int _mode)
			{
				close();

				path = strdup (_path);
				mode = _mode;
				
				fd = ::open (path, mode, 0664);
				if (fd == -1)
					throw IOError (errno) << "opening " << path << " mode " << mode;
			}

		void close()
			{
				if (fd != -1)
					::close (fd);

				if (path)
					free (path);

				fd = -1;
				path = 0;
			}

		void read (void * dest, int n)
			{
				if (::read (fd, dest, n) != n)
					throw IOError (errno) << "reading " << n << " from " << path;
			}
			
		void write (const void * src, int n)
			{
				if (::write (fd, src, n) != n)
					throw IOError (errno) << "writing " << n << " to " << path;
			}
			
		off_t seek (off_t to, int whence)
			{
				off_t r = ::lseek (fd, to, whence);

				if (r == -1)
					throw IOError (errno) << "seeking " << (long long) to 
																<< " from " << whence;
					
				return r;
			}
			
		off_t tell()
			{
				return seek (0, SEEK_CUR);
			}
			
		bool read_four (char * sign)
			{
				char data[4];
				read (data, 4);
				return !strncmp (data, sign, 4);
			}
		
		template <class T>
		T read_le()
			{
				char data [sizeof (T)];
				read (data, sizeof (T));

				return le2host (*((T *) data));
			}
		
		template <class T>
		T read_be()
			{
				char data [sizeof (T)];
				read (data, sizeof (T));

				return be2host (*((T *) data));
			}

		template <class T>
		void write_le (T t)
			{
				t = host2le (t);
				write (&t, sizeof (T));
			}
		
		template <class T>
		void write_be (T t)
			{
				t = host2be (t);
				write (&t, sizeof (T));
			}
};

#endif /* !_BINARY_FILE_H_ */
