/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _CHAIN_H_
#define _CHAIN_H_

#include <stdlib.h>
#include <string.h>

#include "basics.h"

class BasicChained
{
	friend class BasicChain;

	protected:
		BasicChained * _next;

	public:
		BasicChained()
			{
				_next = 0;
			}
};

template <class T>
class Chained
:	public BasicChained
{
	friend class BasicChain;

	public:
		Chained() : BasicChained() 
			{ }

		T *		
			get_next()
				{
					return (T *) _next;
				}
};

class BasicChain
{
	protected:
		BasicChained *	first;

	public:
		BasicChain()
			{
				first = 0;
			}

		void				
			prepend (BasicChained * c)
				{
					if (!first) {
						first = c;
						return;
					}

					c->_next = first;

					first = c;
				}
		
		void	
			remove (BasicChained * c)
				{
					if (first == c) {
						first = c->_next;
						return;
					}
				
					BasicChained * p = first;

					while (p && p->_next != c)
						p = p->_next;

					if (p->_next != c)
						throw AssertionError() << "chain assertion failed.";

					p->_next = c->_next;
				}
};

template <class T>
class Chain
:	public BasicChain
{
	public:
		Chain()
			: BasicChain()
			{}

		T *	
			get_first()
				{ 
					return (T *) first;
				}

		T *	
			get_next (T * c)
				{
					return (T *) ((BasicChained *) c)->next;
				}
};

#endif /* _CHAIN_H_ */
