/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _DECYCLER_H_
#define _DECYCLER_H_

class DecycledAudio
: public AudioSignal
{
	public:
		int played, used;

	public:
		DecycledAudio (int size = 0)
			: AudioSignal (size, DecycledAudio_type)
			{ 
				reset();
			}
	
		/* sets played = used = 0. */
		void reset();
		
		/* copies up to 'frames' frames into 'dest', returning the number of 
		 * frames copied and advances 'played'.
		 */
		int play (d_sample * dest, int frames);
		
		/* copies up to 'frames' from 'src', returning the number of frames
		 * copied and advances 'used'.
		 */
		int record (d_sample * src, int frames);
};

/* Decycler collects CyclicAudio sample data into DecycledAudio,
 * pushing the sample when enough data has been collected.
 */
class Decycler
: public Plugin
{
	public:
		DecycledAudio * current;
		int collect;

	public:
		Decycler();
		~Decycler();
		
		void init (int collect);

		/* Plugin:: */
		virtual void set_state (Graph *, int state);
		virtual void process_audio (int frames);
};

/* Recycler streams DecycledAudio data back into CyclicAudio events.
 */
class Recycler
: public Plugin
{
	public:
		/* measured difference between original sample time and playback time. */
		int latency;

		DecycledAudio * current;

	public:
		Recycler();
		~Recycler();

		/* Plugin:: */
		virtual void set_state (Graph *, int state);
		virtual void process_audio (int frames);
};

#endif /* _DECYCLER_H_ */
