/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _DISKSTREAM_H_
#define _DISKSTREAM_H_

#include "Thread.h"
#include "Event.h"
#include "Graph.h"

/* more than 4 SoundFileSegments don't seem to make sense. */
#define MAX_SEGMENTS	4		

class SoundFile;
class SoundFileSegment;

class DiskStream
: public Plugin
{
	public:
		DiskStream();
		~DiskStream();

		d_frame offset;			/* starting point in graph time. */
		d_frame next_frame;	/* assign this frame to the next free segment. */

		SoundFile * file;

		char * path;
		int channels;
		int sample_rate;
		
		int monitor;
		int to_play_on_loop;
		int to_play_on_seek;

		atomic mode;
		enum {
			Read = 1,
			Write = 2,
			ReadWrite = 3, /* only temporarily, while switching to Read mode. */
		};

		/* do this once only. */
		void set_file (SoundFile * file);
		
		/* Plugin:: */
		virtual void pre_insert (Graph *);
		virtual void set_state (Graph *, int);

		virtual void process_audio (int frames);
		virtual void process_disk (Plugin *);
	
		virtual void seek();

		virtual void pre_remove_other (Plugin * p)
			{
				if (DiskScheduler_type->is_derived (p))
					throw RuntimeError() << this << " needs " << p;
			}

	protected:
		SoundFileSegment * segments [MAX_SEGMENTS];

		void reset_segments (d_frame f);
		void flush();

		void request_io();
		atomic requested;	/* >= 0 : we have requested io. */
		atomic short_warning;	/* >= 0: we have warned of low water. */
		
		d_frame next_segment (d_frame); /* calculate next segment start */

		void process_read (int frames);
		void process_write (int frames);

		/* read ahead based on frame position */
		void process_disk_read (d_frame f);
		void process_disk_write();

		/* do we have a segment ready for frame f? */
		bool have_segment (d_frame f);
};

#endif /* _DISKSTREAM_H_ */
