/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _FIFO_H_
#define _FIFO_H_

template <class T>
class Fifo
{
	private:
		T * stack;
		int size, r, w, used; /* read and write pointers */

	public:
		Fifo (int _size)
			{
				stack = 0;
				size = _size;
				used = 0;
				r = w = 0;
				
				if (size) alloc (size);
			}
		
		~Fifo()
			{
				if (stack) free (stack);
			}

		void alloc (int _size)
			{
				if (stack) throw AssertionError() << "already allocated.";

				size = _size;
				stack = (T *) calloc (size, sizeof (T));
				if (!stack) throw MemoryError();
			}
			
		int get_size() { return size; }
		int have() { return used; }
		bool is_full() { return used == size; }

		void push (T t)
			{
				if (used == size) throw OverflowError();

				stack[w] = t;
				w = (w + 1) % size;
				++used;
			}

		T pop()
			{
				if (used == 0) throw EOFError();
				
				T t = stack[r];
				r = (r + 1) % size;
				--used;

				return t;
			}

		T peek()
			{
				if (used == 0) return 0;
				return stack[r];
			}
};

#endif /* _FIFO_H_ */
