/* (c) 2001-4 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

/* .pat file loader and voice storage */

#ifndef _GUS_VOICE_H_
#define _GUS_VOICE_H_

class GUSVoice
: public NamedObject
{
	public:
		AudioSignal * sample;
		
		enum {
			TwoByteSamples = 1 << 0,
			Unsigned = 1 << 1,
			Looped = 1 << 2,
			PingPong = 1 << 3,
			Reverse = 1 << 4,
			Sustain = 1 << 5,
			Envelope = 1 << 6,
			ClampRelease = 1 << 7,
		};
		byte mode;

		int sample_rate;

		struct {
			double start, end;
		} loop;
		
		struct {
			int low, high, root;
		} freq;

		struct {
			int rates[6], offsets[6];
		} envelope;

		int pan;
		int note;
		
		GUSVoice();
		~GUSVoice();
};

class GUSPatchFile
{
	public:
		int fd;
		TypedObjectList<GUSVoice> patches;

		GUSPatchFile (char * path)
			: patches ("patches", 32)
			{
				fd = open (path, O_RDONLY);
				if (fd == -1)
					throw OSError (errno) << path;
			}

		~GUSPatchFile()
			{
				if (fd != -1)
					close (fd);
			}
		
		void read (void * b, int size)
			{
				if (size != ::read (fd, b, size))
					throw OSError (errno);
			}

		void skip (int size)
			{
				if (lseek (fd, size, SEEK_CUR) == -1)
					throw OSError (errno);
			}

		void read_patch();

		uint8 read_8()
			{
				uint8 b;
				read (&b, sizeof (uint8));
				return b;
			}

		uint16 read_16()
			{
				uint16 s;
				read (&s, sizeof (uint16));
				return le2host (s);
			}

		uint32 read_32()
			{
				uint32 i;
				read (&i, sizeof (uint32));
				return le2host (i);
			}
};

#endif /* _GUS_VOICE_H_ */
