/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _GAIN_H_
#define _GAIN_H_

#include "Plugin.h"

extern Type * Gain_type;
extern Type * ADSR_type;

class SoundingNotes
{
	public:
		byte notes[128];
		int sounding;

		SoundingNotes()
			{
				memset (notes, 0, 128);
				sounding = 0;
			}

		void note_off (int note)
			{
				if (notes[note] == 0)
					return;
				
				--notes[note];
				--sounding;
			}

		void note_on (int note)
			{
				++notes[note];
				++sounding;
			}
};

class Gain
: public Plugin
{
	protected:
		/* the actual factor */
		d_float factor;

		/* the target factor */
		d_float state;

		void end_cycle (CyclicAudio *);

	public:
		d_sample peak;
		bool do_peak;
		
		Gain (Type * t = Gain_type, d_sample g = 1.0, bool p = true);
		
		void init (int n_outs);
		
		/* getsets */
		void set_factor (d_float f) 
			{ state = f; }

		d_float get_factor() 
			{ return state; }

		/* Plugin::* */
		void process_audio (int frames);
};

class StereoGain
: public Gain
{
	public:
		StereoGain();
		
		void init();

		void process_audio (int frames);
};

class ADSR
: public Gain
{
	protected:
		enum {
			Off = 0,
			Attack = 1,
			Decay = 2,
			Sustain = 3,
			Release = 4
		};
		int phase;
		d_sample rate;
		
		d_sample velocity;
		
		In * notes_in;
		Out * notes_out;
		Out * cv_out;

		SoundingNotes notes;

		/* fetch notes, update state and apply. */
		void one_cycle (d_sample *, int frames, d_time delta, d_tick tick);
				
	public:
		d_sample a, d, s, sr, r;

		ADSR();

		/* Plugin::* */
		void process_audio (int frames);
};

#endif /* _GAIN_H_ */
