/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _GEOMETRY_H_
#define _GEOMETRY_H_

template <typename T>
class Vector
{
	public:
		T x, y;

		Vector() {}
		
		Vector (T _x, T _y) 
			{
				x = _x;
				y = _y;
			}
		
		Vector (const Vector<T> & other) 
			{
				*this = other;
			}

		Vector & operator = (const Vector<T> & that) 
			{
				x = that.x;
				y = that.y;

				return *this;
			}

		bool operator == (const Vector<T> & that) 
			{
				return that.x == x && that.y == y;
			}

		/* not the length -- call sqrt if you need it. */
		operator T () 
			{
				return x * x + y * y;
			}

		Vector & operator += (const Vector<T> & that) 
			{
				x += that.x;
				y += that.y;

				return *this;
			}

		Vector & operator -= (const Vector<T> & that) 
			{
				x -= that.x;
				y -= that.y;

				return *this;
			}
};

template <typename T>
class Rect
{
	public:
		T x, y, width, height;
		
		Rect() 
			{ 
				x = y = width = height = 0;
			}
		
		Rect (const Rect<T> & that) 
			{
				*this = that;
			}
		
		Rect (Vector<T> origin, Vector<T> size) 
			{
				set_from (origin.x, origin.y, size.x, size.y);
			}

		Rect (T _x, T _y, T _width, T _height) 
			{
				set_from (_x, _y, _width, _height);
			}

		Rect & set_from (T _x, T _y, T _width, T _height) 
			{
				x = _x; 
				y = _y, 
				width = _width, 
				height = _height;
				return *this;
			}
			
		Rect (PyObject * arg);

		bool has (T _x, T _y)
			{
				return _x >= x && _x < x + width && _y >= y && _y < y + height;
			}

		operator T () 
			{
				return width * height;
			}

		bool operator == (const Rect<T> & that) 
			{	
				return x == that.x && y == that.y &&
						width == that.width && height == that.height;
			}

		Rect<T> & operator = (const Rect<T> & that) 
			{	
				set_from (that.x, that.y, that.width, that.height);
				return *this; 
			}

		Rect<T> & operator += (const Rect<T> & that) 
			{
				T fx = min (x, that.x);
				T fx1 = max (x + width, that.x + that.width);

				T fy = min (y, that.y);
				T fy1 = max (y + height, that.y + that.height);

				return set (fx, fy, fx1 - fx, fy1 - fy);
			}

		Rect<T> & operator += (const Vector<T> & that) 
			{
				x += that.x;
				y += that.y;
				return *this;
			}

		Rect<T> & operator -= (const Vector<T> & that) 
			{
				x -= that.x;
				y -= that.y;
				return *this;
			}

		bool clip (Rect<T> & r) 
			{
				T rx1 = min (r.x + r.width, x + width);
				T ry1 = min (r.y + r.height, y + height);

				r.x = max (r.x, x);
				r.y = max (r.y, y);

				r.width = max (0, rx1 - r.x);
				r.height = max (0, ry1 - r.y);

				return (r.width > 0) && (r.height > 0);
			}
};

#endif /* _GEOMETRY_H_ */
