/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifdef WITH_JACK
#ifndef _JACK_H_
#define _JACK_H_

#include "Plugin.h"
#include "AudioClock.h"
#include "Graph.h"

#include <jack/jack.h>

class JackIn;
class JackOut;

class Jack
: public AudioClock
{
	private:
		/* when jackd changes sample rate or frames/cycle. */
		void restart_graph();

		/* store the jack_process() PythonThread so we can shut down cleanly. */
		PythonThread * py_thread;
		
	public:
		jack_client_t * handle;
		int jackd_is_down;

		int sample_rate, buffer_size;

		int jack_process (int);
		int jack_buffer_size (int);
		int jack_sample_rate (int);
		void jack_shutdown();
		
	public:
		Jack();
		~Jack();

		/* sets up the connection. */
		void init (char * name);
		
		/* asks jackd to connect/disconnect 'source' and 'dest'. */
		void connect (const char * source, const char * dest);
		void disconnect (const char * source, const char * dest);

		/* Plugin::* */
		virtual void pre_insert (Graph *);
		virtual void set_state (Graph *, int _state);
};

class JackIn
: public Plugin,
	public Cache
{
	public:
		PointerList<jack_port_t> ports;
		TypedObjectList<CyclicAudio> events;
	
		JackIn();
		~JackIn();
		
		/* sets up the ports. */
		void init (Jack *, int n_ports);

		void add_port (jack_port_t * port);

		/* Cache::* */
		virtual void push (Object *);
		
		/* Plugin::* */
		virtual void set_state (Graph * g, int s)
			{
				if (g->audio_clock == 0)
					throw AssertionError() << "no Jack in the graph.";
				if (!Jack_type->is_derived (g->audio_clock))
					throw AssertionError() << "need a Jack in the graph.";

				Plugin::set_state (g, s);
			}

		virtual void post_remove_other (Plugin * p)
			{
				if (Jack_type->is_derived (p))
					state = Off;
			}

		virtual void process_audio (int frames);

		/* override default plugin behaviour, which is to reallocate audio
		 * buffers in the event cache. Since the Jack shm buffers are used,
		 * this must be prevented.
		 */
		virtual void set_frames_per_cycle (int frames_per_cycle) { }
};

class JackOut
: public Plugin
{
	public:
		PointerList<jack_port_t> ports;

		JackOut();
		~JackOut();

		/* sets up the ports. */
		void init (Jack *, int n_ports);

		void add_port (jack_port_t * port);

		/* Plugin::* */
		virtual void set_state (Graph * g, int s)
			{
				if (g->audio_clock == 0)
					throw AssertionError() << "no Jack in the graph.";
				if (!Jack_type->is_derived (g->audio_clock))
					throw AssertionError() << "need a Jack in the graph.";

				Plugin::set_state (g, s);
			}

		virtual void post_remove_other (Plugin * p)
			{
				if (Jack_type->is_derived (p))
					state = Off;
			}

		virtual void process_audio (int frames);
};

#endif /* _JACK_H_ */
#endif /* WITH_JACK */
