/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _LADSPA_PLUGIN_H_
#define _LADSPA_PLUGIN_H_

#include "ladspa.h"
#include "Plugin.h"

#define LADSPA_DEFAULT_PATH "/usr/local/lib/ladspa/"

class LadspaPlugin;

extern Type * LadspaPlugin_type;
extern Type * LadspaOut_type;
extern Type * LadspaIn_type;

class LadspaPort
{
	friend class LadspaPlugin;
	private:
		CyclicAudio * event;

	protected:
		LADSPA_Data value, old_value;

	public:
		int index;
		int hints;
		LADSPA_Data bounds[2], sample_rate;

		LadspaPort() { event = 0; }
		LadspaPort (const LADSPA_Descriptor * d, int i);

		void set_value (LADSPA_Data v);
		LADSPA_Data get_value();
};

class LadspaOut;

class LadspaIn
: public In, 
	public LadspaPort
{
	public:
		LadspaIn() { }
		LadspaIn (LadspaPlugin * p, const LADSPA_Descriptor * d, int i);

		/* read incoming events until tick and set LadspaPort::value. */
		void event_cycle (d_tick tick);
};

class LadspaOut
: public Out, 
	public LadspaPort
{
	public:
		LadspaOut() { }
		LadspaOut (LadspaPlugin * p, const LADSPA_Descriptor * d, int i);

		/* emit events if LadspaPort::value has changed. */
		void event_cycle (d_tick tick);
};

class LadspaPlugin
:	public Plugin
{
	public:
		char * dlpath;
		void * dlhandle;
		void * handle;
		int audio_ports;

		PointerList<LadspaPort> ports;

		LADSPA_Descriptor_Function get_descriptor;

		void setup();

	public:
		const LADSPA_Descriptor *	descriptor;
		int index;

		/* whether to divide the processing cycle into subcycles for better
		 * event response. */
		bool subcycle;

		LadspaPlugin();
		~LadspaPlugin();
		
		/* to set up, first open, then call one of the two inits. */
		void open (const char * path);
		void init (int id);
		void init (char * name);

		/* or init the next from an already loaded library. */
		void init (LadspaPlugin * p);

		void one_shot (int, AudioSignal **);

		/* Plugin::* */
		virtual void set_state (Graph *, int state);
		virtual void process_audio (int frames); 

		virtual void pre_insert (Graph * g) 
			{
				if (handle)
					throw AssertionError() << "not a virgin anymore.";
			}

		void instantiate (int fs);

	protected:
		void reconnect_audio (int offset);

		void cleanup();
		void connect (LadspaPort *, LADSPA_Data * = 0);
};

#endif /* _LADSPA_PLUGIN_H_ */
