/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>

#include "basics.h"

#include "List.h"

BasicIterator::BasicIterator()
{
	list = 0;
	index = 0;
}

BasicIterator::BasicIterator (BasicIterator & it)
{
	list = 0;
	init (it.list, it.index);
}

bool
BasicIterator::valid()
{ 
	return index >= 0 && index < list->length; 
}

void
BasicIterator::init (List * l, int i)
{
	if (list) list->remove_iterator (this);

	list = l;
	index = i;

	if (list) list->add_iterator (this);
}

BasicIterator::~BasicIterator()
{
	if (list) list->remove_iterator (this);
}

BasicIterator::operator int()
{ 
	if (list == 0)
		throw AssertionError() << "nothing to iterate.";

	if (index < 0 || index > list->length) 
		return list->length;
	return index;
}

void *
BasicIterator::resolve()
{ 
	if (list == 0)
		throw AssertionError() << "nothing to iterate.";

	int i = (int) (*this);
		
	if (i >= list->length)
		return 0;

	return list->item (index); 
}

int
BasicIterator::have()
{
	return list->length - index;
}

/*
		List::stuff
*/

void
List::clear()
{
	basic_pre_remove (0, length);
	length = 0;

	if (allocated > granularity) 
		reallocate (granularity);
}

void
List::reallocate (int n)
{
	if (items == 0)
		throw AssertionError() << "nothing allocated yet.";

	int a = n + (granularity - (n % granularity));

	if (a == allocated)
		return;

	allocated = a;
	
	items	= (void *) realloc (items, size * allocated);
}

void
List::copy_items (const void * data, int n)
{
	reallocate (length = n);
	memcpy (items, data, length);
}

void *				
List::forget_allocation()
{
	allocated = 0;
	length = 0;

	void * was = items;
	items = 0;

	return was;
}

int
List::insert (int n, int i)
{
	reallocate (length + n);

	if (i >= 0 && i < length)
	{
		/* need to move items */
		length += n;
		memmove	(item (i + n), item (i), (length - i) * size);
	}
	else
	{
		if (i < 0 || i >= length)
			/* append */
			i = length;
		length += n;
	}

	return i;
}

int
List::insert_items (const void * data, int n, int i)
{
	if (n <= 0) 
		throw ValueError() << "cannot insert 0 or less items";
	
	i = insert (n, i);

	memcpy (item (i), data, n * size);
	basic_post_insert (i, n);
	
	return i;
}

bool
List::remove_items (int i, int n)
{
	if (n <= 0) 
		throw ValueError() << "cannot remove 0 or less items";

	int end = i + n;

	if (i < 0 || i >= length)
		throw AssertionError() << i << "not in range (0, " << length << ")";
	if (end < 0 || end > length)
		throw AssertionError() << end << "not in range (0, " << length + 1 << ")";

	basic_pre_remove (i, n);

	if (length - end)
		memmove (item (i), item (end), (length - end) * size);

	reallocate (length = length - n);

	return true;
}


void
List::swap (int i0, int i1)
{
	void * helper = malloc (size);

	memcpy (helper, item (i0), size);
	memcpy (item (i0), item (i1), size);
	memcpy (item (i1), helper, size);

	free (helper);
}

void
List::move (int from, int to)
{
	if (from == to)
		return;

	if (from < 0 || from >= length)
		throw AssertionError() << from << "not in range (0, " << length << ")";
	if (to < 0 || to >= length)
		throw AssertionError() << to << "not in range (0, " << length + 1 << ")";

	void * tmp = malloc (size);
	memcpy (tmp, item (from), size);
	
	if (to > from)
		memmove (item (from), item (from + 1), (to - from) * size);
	else
		memmove (item (to + 1), item (to), (from - to) * size);

	memcpy (item (to), tmp, size);
	
	free (tmp);
}


