/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"

#include "Modulator.h"
#include "PyType.h"

Type * Modulator_type;

Modulator::Modulator()
: Plugin (Modulator_type, Audio)
{
	add = 0.0;
	mul = 1.0;

	cv_in = Plugin::add_in (new In (this, FloatEvent_type, IN_SLOTS, "cv dry in"));
	add_in = Plugin::add_in (new In (this, FloatEvent_type, 1, "add"));
	mul_in = Plugin::add_in (new In (this, FloatEvent_type, 1, "mul"));
	Plugin::add_out (new Out (this, FloatEvent_type, "cv dry out"));
	Plugin::add_out (new Out (this, FloatEvent_type, "cv wet out"));
}

void
Modulator::process_audio (int frames)
{
	d_frame f = graph->frame;
	d_tick tick;

	while (frames > 0)
	{
		int n = min (64, frames);

		bool is_loop = graph->is_loop_frame (f, f + n);

		tick = graph->frame2tick (f + n);

		int changed;

		changed  = cv_in->pop_value (tick, &value);
		changed |= add_in->pop_value (tick, &add);
		changed |= mul_in->pop_value (tick, &mul);

		if (changed)
		{
			FloatEvent * dry = (FloatEvent *) outs[0]->cache->pop();
			FloatEvent * wet = (FloatEvent *) outs[1]->cache->pop();
			
			dry->value = value;
			wet->value = dry->value * mul + add;
			wet->tick = dry->tick = tick;

			outs[0]->push (dry);
			outs[1]->push (wet);
		}

		frames -= n;
		f += n;

		if (is_loop)
			++loop_cycle,
			f -= graph->loop.frame[1] - graph->loop.frame[0];
	}
}

/* /////////////////////////////////////////////////////////////////////// */

static int
Modulator_set_add (Modulator * m, PyObject * value)
{
	if (!PyArg_Parse (value, "d", &m->add))
		return -1;
	return 0;
}

static PyObject *
Modulator_get_add (Modulator * m)
{
	return PyFloat (m->add);
}

static int
Modulator_set_mul (Modulator * m, PyObject * value)
{
	if (!PyArg_Parse (value, "d", &m->mul))
		return -1;
	return 0;
}

static PyObject *
Modulator_get_mul (Modulator * m)
{
	return PyFloat (m->mul);
}

static PyGetSetDef 
Modulator_getsets [] = {
	{"add", (getter) Modulator_get_add, (setter) Modulator_set_add,
	 "the value that is added to incoming signal, after multiplying it."},
	{"mul", (getter) Modulator_get_mul, (setter) Modulator_set_mul,
	 "the value the incoming signal is multiplied with, before adding 'add'."},
	{ }
};

char * Py<Modulator>::__doc__ = 
"Modulator ([mul = 1.0, add = 0.0]) -- "
"a plugin that operates in the audio cycle. The incoming FloatEvent "
"signal is multiplied by the 'mul' value, then 'add' is added.\n"
"NB: This plugin will probably be restructured or dumped in future releases. "
"Your design ideas for a hypothetical Translator plugin that maps from one "
"unit space to another are welcome.";

void
Py<Modulator>::type_init (Type * t, PyObject * dict)
{
	t->tp_base = Plugin_type;
	t->tp_getset = Modulator_getsets;
	t->tp_flags |= Py_TPFLAGS_BASETYPE;
}

void
Py<Modulator>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {"mul", "add", 0};

	if (!PyArg_ParseTupleAndKeywords (args, kwds, "|dd", keys,
				&mul, &add))
		throw blewit();
}

/* /////////////////////////////////////////////////////////////////////// */

void
init_Modulator_types (PyObject * dict)
{
	Modulator_type = new PyType<Modulator> ("Modulator", dict);
}

