/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"

#ifdef WITH_MPEG3
#include "MpegFile.h"
#include "PyType.h"

static Type * MpegFile_type;

MpegFile::MpegFile()
: SoundFile (MpegFile_type)
{
	file = 0;
	path = 0;
	stream = 0;
	at = 0;
}

MpegFile::~MpegFile()
{
	if (file)
		mpeg3_close (file);
		
	if (path)
		free (path);
}

void
MpegFile::open (char * _path, int _stream)
{
	if (file)
		throw AssertionError() << "already opened " << path;

	file = mpeg3_open (_path);
	
	if (file == 0)
		throw IOError (errno) << "opening supposed mpeg file " << _path;
	
	path = strdup (_path);
	
	int streams = mpeg3_total_astreams (file);
	if (streams == 0)
		throw IOError() << path << " has no audio streams.";
	else if (_stream >= streams)
		throw IOError() << path << " has only " << streams << " audio streams.";
		
	stream = _stream;

	channels = mpeg3_audio_channels (file, stream);
	if (channels <= 0)
		throw IOError() << path << " stream " << stream << " has no channels.";

	sample_rate = mpeg3_sample_rate (file, stream);
	frames = mpeg3_audio_samples (file, stream);
}

void
MpegFile::seek (d_frame f)
{
	if (at == f)
		return;

	if (f > LONG_MAX)
		throw AssertionError() << "libmpeg3 can do 32 bit offsets only.";

	if (mpeg3_set_sample (file, (long) f, stream) == -1)
		throw IOError() << path << " is not seekable.";
}

d_frame
MpegFile::tell()
{
	return at;
}

void
MpegFile::read (d_sample ** dest, int frames)
{
	int r = mpeg3_read_audio (file,
			dest[0], (short *) 0,
			0, /* channel */
			frames,
			stream);

	if (r == -1)
		throw IOError() << "reading channel 0 from " << path;

	for (int i = 1; i < channels; ++i)
	{
		int r = mpeg3_reread_audio (file,
				dest[i], (short *) 0,
				i,
				frames,
				stream);

		if (r == -1)
			throw IOError() << "reading channel " << i << " from " << path;
	}

	at += frames;
}

/* //////////////////////////////////////////////////////////////////////// */

char * Py<MpegFile>::__doc__ = 
"MpegFile ('/path/file.mp3'[, stream=0]) -- "
"a read-only file that is accessed through libmpeg3. "
"Use in conjunction with a DiskStream. This plugin is quite demanding "
"in terms of CPU usage.";

void
Py<MpegFile>::type_init (Type * t, PyObject * dict)
{
	t->tp_base = SoundFile_type;
}

void
Py<MpegFile>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {"path", "stream", 0};
	char * path;
	int stream = 0;

	if (!PyArg_ParseTupleAndKeywords (args, kwds, "s|i", keys, 
			&path, &stream))
		throw blewit();
	
	this->open (path, stream);
}

#endif /* WITH_MPEG3 */

void
init_MpegFile_types (PyObject * dict)
{
	#ifdef WITH_MPEG3
	MpegFile_type = new PyType<MpegFile> ("MpegFile", dict);
	#endif
}
