/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _PART_H_
#define _PART_H_

#include "EventList.h"
#include "Plugin.h"
#include "NoteOffs.h"

extern Type * MidiPlexer_type;

class Part
: public Plugin
{
	protected:
		Out * out;

		/* enqueue events, called by process_beat. */
		void enqueue (d_tick until);
		inline void enqueue_one (Event * e);

		/* only used if type of events is or is derived from MidiEvent_type. */
		NoteOffs note_offs;

	public:
		EventList * list;
		d_tick tick;
		d_tick delay;
		int mute;
		int channel; /* -1 = don't impose a MIDI channel */
		int transpose; /* semi tones */
		int bank, program; /* -1 = don't send */
		int volume; /* -1 = don't send */

		Part();
		~Part();

		void set_list (EventList *);

		/* Plugin:: */
		virtual void set_state (Graph *, int state);
		virtual void seek();

		virtual void process_beat (d_tick until);
	
	protected:
		void queue_note_off (Note * n);
};

class MidiPlexer
: public Plugin
{
	public:
		In * in;
		Out * thru, * copy;
		Mutex mutex;

	public:
		int channel; /* -1 = don't impose a MIDI channel */
		byte cc_map[128]; /* controller mapping */
		int block_mask; /* forward pitch bend events */
		
		int status_mask (byte status)
			{ return 1 << ((status & 0x70) >> 4); }
		bool is_blocked (byte status)
			{ return block_mask & status_mask (status); }
		void block (byte status)
			{ block_mask |= status_mask (status); }
		void unblock (byte status)
			{ block_mask &= ~status_mask (status); }
				
		MidiPlexer();

		/* Plugin:: */
		virtual void process_trigger (In * in);
};

#endif /* _PART_H_ */
