#ifndef _PHASE_VOCODER_H_
#define _PHASE_VOCODER_H_

#ifdef WITH_FFTW

#include <basics.h>
#include <Plugin.h>

#include "pvpp.h"

extern Type * PVocFrame_type;
extern Type * PhaseVocoder_type;

class PVocFrame
: public Object
{
	public:
		d_frame frame;
		pvoc_frametype frametype;

		int size;
		d_sample * data;

	public:
		PVocFrame (int s = 0, Type * t = PVocFrame_type)
			: Object (t)
			{ 
				frame = 0;
				frametype = PVOC_INVALID;
				size = s;
				data = 0;
				
				if (size)
					set_size (size);
			}

		PVocFrame (PVocFrame & e)
			: Object (PVocFrame_type)
			{
				data = 0;
				size = 0;
				set_size (e.size);
			
				*this = e;
			}

		~PVocFrame()
			{
				if (data) free (data);
			}

		void set_size (int n)
			{
				if (n == size && data)
					return;

				if (data) 
					free (data);

				size = n;
				data = (d_sample *) calloc (size, sizeof (d_sample));
	
				if (data == 0) 
					throw MemoryError() << "allocating a sample buffer of size " << n;
			}
};

class PhaseVocoder
: public Plugin
{
	public:
		phasevocoder vocoder;

		int sample_rate;
		int fft_length;
		int decfac;

		int direction; /* >= 0 in / < 0 out */
		int fill; /* how many audio samples we have fed the vocoder, runs
							 * from 0 to decfac */
		
		pvoc_frametype frametype;

		PhaseVocoder();
		~PhaseVocoder();

		void init (int direction, int sample_rate, int fft_length, int decfac);

		void set_state (Graph * g, int state);
		void process_audio (int frames);

		int produce_frame (d_sample * s, int n, PVocFrame * frame);
		int produce_audio (PVocFrame * frame, d_sample * s);
};

#endif /* WITH_FFTW */

#endif /* _PHASE_VOCODER_H_ */
