/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _PIPE_H_
#define _PIPE_H_

#include "Pollable.h"

class Pipe
: public Pollable
{
	public:
		int fd[2];
		char * name;
		Object * object;

		enum {
			Read = 0,
			Write = 1
		};

		Pipe (char * n, Object * o = 0, bool block = 0)
			{
				name = n;
				object = o;

				if (pipe (fd))
					throw OSError (errno) << "creating pipe '" << name << "'.";
				
				if (!block)
				{
					set_nonblocking (Pipe::Read);
					set_nonblocking (Pipe::Write);
				}
			}

		virtual ~Pipe()
			{
				close (fd[0]);
				close (fd[1]);
			}

		void set_nonblocking (int i)
			{
				if (fcntl (fd[i], F_SETFL, O_NONBLOCK) == -1)
					throw OSError (errno) << "setting nonblocking on fd " << i;
			}

		inline int write (void * x, int n)
			{
				int w = ::write (fd[1], x, n);
				
				if (w <= 0)
					throw OSError (errno) << "writing to pipe '" << name << "'.";
				
				return w;
			}

		inline int read (void * x, int n)
			{
				return ::read (fd[0], x, n);
			}

		inline void write_trigger (int trigger)
			{
				if (::write (fd[1], &trigger, sizeof (int)) == sizeof (int))
					return;
				
				throw OSError (errno) 
						<< "writing trigger " << trigger 
						<< " to pipe '" << name << "'.";
			}

		inline int read_trigger()
			{
				int trigger;

				if (::read (fd[0], &trigger, sizeof (int)) == sizeof (int))
					return trigger;
				
				throw OSError (errno) 
						<< "reading trigger from pipe '" << name << "'.";
			}

		virtual void get_pollfd (InstanceList<struct pollfd> & list)
			{
				int i = list.insert (1);

				list[i].fd = fd[0];
				list[i].events = POLLIN | POLLERR;
			}
	
		virtual operator Object * ()
			{
				return object;
			}
};

#endif /* _PIPE_H_ */
