/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _POLLABLE_H_
#define _POLLABLE_H_

/* basic interface for objects that want notification of incoming data. 
 * see MidiInThread. 
 */
class Pollable
{
	public:
		virtual void get_pollfd (InstanceList<struct pollfd> & list) = 0;

		virtual operator Object * ()
			{
				return 0;
			}

		virtual void do_io() 
			{ }
};

class Poll
: public InstanceList<struct pollfd>
{
	public:
		Poll (const char * name)
			: InstanceList<struct pollfd> (name)
			{ }

		int poll (int mseconds, int l = -1)
			{
				if (length == 0)
					throw AssertionError() << "nothing to poll.";

				if (l < 0) l = length;

				for (int i = 0; i < length; ++i)
					(*this)[i].revents = 0;

				int r = ::poll ((struct pollfd *) items, l, mseconds);

				if (r < 0)
				{
					if (errno == EINTR)
						msg(), "EINTR polling ", name;
					else
						throw IOError (errno) << "in poll().";
				}

				for (int i = 0; i < length; ++i)
				{
					if ((*this)[i].revents & POLLERR)
						throw IOError() << "poll error.";
					if ((*this)[i].revents & POLLHUP)
						throw IOError() << "poll: hung up.";
				}

				return r;
			}

		void add (Pollable & p)
			{
				p.get_pollfd (*this);
			}

		void add (int fd, int events)
			{
				struct pollfd pfd = {fd, events};
				insert_item (pfd);
			}
};

#endif /* _POLLABLE_H_ */
