/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _PY_H_
#define _PY_H_

#include "basics.h"
#include "List.h"

template <class O> class Py
:	public O
{
	public:
		/* called at module initialization, this sets base object and getsets, 
		 * methods and other PyTypeObject properties. */
		static void type_init (Type *, PyObject * dict);

		/* none without. */
		static char * __doc__;
		
		/* same goes for instance initialization. */
		void __init__ (PyObject * args, PyObject * kwds);
};

/* /////////////////////////////////////////////////////////////////////// */

/* these two embrace a function call from python. for our convenience, they
 * allow us to simply throw a C++ exception at python.
 */
#define Py_TRY \
{ try  
		
#define Py_CATCH(err) \
	catch (blewit & b) \
	{ \
		if (!PyErr_Occurred()) \
			b.throw_at_python(); \
	} \
	return err; \
}

/* /////////////////////////////////////////////////////////////////////// */

extern d_note PyNote2n (PyObject *);

inline PyObject * 
PyError (PyObject * exc, const char * s)
{
	PyErr_SetString (exc, s);
	return NULL;
}

inline PyObject * 
PyNone()
{
	Py_INCREF (Py_None);
	return Py_None;
}

inline PyObject * 
PyInt (long l)
{
	return PyInt_FromLong (l);
}

inline PyObject * 
PyFrame (d_frame f)
{
	return PyLong_FromLongLong (f);
}

inline PyObject * 
PyFloat (double d)
{
	return PyFloat_FromDouble (d);
}

inline PyObject *
PyString (const char * s)
{
	return PyString_FromString (s ? s : "");
}

inline PyObject *
PyString (void * s, int n)
{
	return PyString_FromStringAndSize ((char *) s, n);
}

inline PyObject *
PyTuple (ObjectList * l)
{
	PyObject * s = PyTuple_New (l->length);
	
	for (int i = 0; i < l->length; i++)
		PyTuple_SetItem (s, i, ::incref ((*l) [i]));

	return s;
}

#endif /* _PY_H_ */
