/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"

#include "Py.h"

Type * Object_type = 0;

char * Py<Object>::__doc__ = 
"Object() -- basetype to all module types.";

void
Py<Object>::type_init (Type * t, PyObject * dict)
{
	t->tp_base = 0;
}

void
Py<Object>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {0};

	if (!PyArg_ParseTupleAndKeywords (args, kwds, "", keys))
		throw blewit();
}

/* ////////////////////////////////////////////////////////////////////// */

static PyObject * 
NamedObject_get_name (NamedObject * o) 
Py_TRY 
{
	return PyString (o->get_object_name());
} 
Py_CATCH (0);

static int
NamedObject_set_name (NamedObject * o, PyObject * value) 
Py_TRY 
{
	if (!PyString_Check (value))
		throw ValueError() << "must be a string.";
	
	o->set_object_name (PyString_AsString (value));

	return 0;
} 
Py_CATCH (-1);

static PyGetSetDef NamedObject_getsets [] = {
	{"name", (getter) NamedObject_get_name, (setter) NamedObject_set_name, 
	 "the object name."},
	{0},
};

/* ////////////////////////////////////////////////////////////////////// */

Type * NamedObject_type = 0;

char * Py<NamedObject>::__doc__ = 
"NamedObject (name) -- an Object that has a mutable name. "
"Most of the 'more complicated' object types inherit from this.";

void
Py<NamedObject>::type_init (Type * t, PyObject * dict)
{
	t->tp_base = Object_type;
	t->tp_getset = NamedObject_getsets;
}

void
Py<NamedObject>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {"name", 0};
	char * name;
	
	if (!PyArg_ParseTupleAndKeywords (args, kwds, "s", keys, &name))
		throw blewit();
	
	/* protect from popping random values from the stack by interpreting '%'. */
	set_object_name ("%s", name);
}

