/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"
#include "ObjectFifo.h"

#include "PyType.h"

/* getsets */
 
static PyObject * 
ObjectFifo_get_size (ObjectFifo * f)
{
	return PyInt (f->get_size());
}

static PyObject * 
ObjectFifo_get_have (ObjectFifo * f)
{
	return PyInt (f->have());
}

static PyGetSetDef 
ObjectFifo_getsets [] = 
{
	{"size", (getter) ObjectFifo_get_size, (setter) 0, 
	 "the number of objects the fifo can hold."},
	{"have", (getter) ObjectFifo_get_have, (setter) 0, 
	 "the number of objects currently stored."},
	{}
};

static PyObject *
ObjectFifo_push (ObjectFifo * self, PyObject * o) 
Py_TRY
{
	if (!self->push_object ((Object *) o))
		throw OverflowError() << "full.";

	::incref (o);
	return PyNone();
} 
Py_CATCH (0);

static PyObject *
ObjectFifo_pop (ObjectFifo * self) 
Py_TRY
{
	Object * o = self->pop_object();

	/* steal the reference to 'o'. */
	return o ? o : PyNone();
} 
Py_CATCH (0);

static PyMethodDef 
ObjectFifo_methods [] = 
{
  {"push", (PyCFunction) ObjectFifo_push, METH_O,
	 "push (object) -- push this object into the fifo."},
	{"pop", (PyCFunction) ObjectFifo_pop, METH_NOARGS,
	 "pop() -- return the bottom-most object and remove it from the fifo."},
	{}
};
	
Type * ObjectFifo_type = 0;

char * Py<ObjectFifo>::__doc__ = 
"ObjectFifo (size[, name]) -- "
"a first-in-first-out queue that is designed for lock-free "
"passage of Objects from one thread to another.";

void
Py<ObjectFifo>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys[] = {"size", "name", 0};
	int size = 0;
	char * name = 0;
	
	if (!PyArg_ParseTupleAndKeywords (args, kwds, "i|s", keys,
	    &size, &name))
		throw blewit();
	
	set_object_name (name);
	alloc (size);
}

/* After the basic initialization has been done in PyType::PyType(), we
 * set additional type properties here. */
 
void
Py<ObjectFifo>::type_init (Type * t, PyObject * dict)
{
	t->tp_base = NamedObject_type;
	t->tp_getset = ObjectFifo_getsets;
	t->tp_methods = ObjectFifo_methods;
}

/* //////////////////////////////////////////////////////////////////////// */

void
init_ObjectFifo_types (PyObject * dict)
{
	ObjectFifo_type = new PyType<ObjectFifo> ("ObjectFifo", dict);
}
