/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _RT_SENTINEL_H_
#define _RT_SENTINEL_H_

#include "Pollable.h"
#include "Pipe.h"

/* RTSentinel runs a sched_fifo thread that waits for life signs. if the
 * wait times out, an always-fail assert is issued to stop the process.
 *
 * the purpose is to prevent a stampedeing audio thread from locking up the 
 * box.
 */

class RTSentinel
: public Thread
{
	public:
		Pipe pipe;
		Poll poll;
		volatile bool active;

		RTSentinel (char * name, int priority)
			: Thread (name),
				pipe ("sentinel pipe"),
				poll ("sentinel poll")
			{
				active = 1;
				poll.add (pipe);
				start_thread_rt (priority);
			}

		enum
		{
			Alive = 'l',
			Quit = 'q'
		};

		void give_livesign (char action = Alive) 
			{
				pipe.write_trigger (action);
			}

		~RTSentinel()
			{
				give_livesign (Quit);

				join_thread();
			}

		virtual void * thread()
			{
				while (1) 
				{
					/* 2 second timeout */
					int r = poll.poll (2000);

					/* timed out, kill this process. */
					if (r == 0 && active)
					{
						fprintf (stderr, "thread %s seems lost in an infinite loop.\n",
								thread_name);
						fprintf (stderr, "trying to invoke gdb.\n");
						
						/* this is pretty tough to do (fork and invoke gdb); the tty is 
						 * probably in raw mode (python -i) but it doesn't seem the time
						 * to be too discerning about the means employed. */
						if (fork())
						{
							sleep (1);
							assert (!"deceased.");
						}
						else
						{
							char cmd[400];
							sprintf (cmd, "sudo gdb python %d", getpid());
							system (cmd);
						}
					}
						
					if (poll[0].revents & POLLHUP)
						throw StopIteration() << "hung up.";
					if (poll[0].revents & ~POLLIN)
						throw IOError() << "poll error.";

					if (r == 1)
					{
						if (pipe.read_trigger() == Quit)
							break;
					}
				}

				return 0;
			}
};

#endif /* _RT_SENTINEL_H_ */
