/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"

#ifdef WITH_SAMPLERATE

#include <samplerate.h>

#include "AudioSignal.h"
#include "PyType.h"

static Type * Resampler_type;

class Resampler
: public Object
{
	public:
		SRC_STATE * src;
		double ratio;
		int error;

		Resampler();
		~Resampler();

		void init (double ratio, int type);
};

Resampler::Resampler()
: Object (Resampler_type)
{
	src = 0;
}

Resampler::~Resampler()
{
	if (src)
		src_delete (src);
}

void
Resampler::init (double r, int type)
{
	src = src_new (type, 1, &error);
	if (src == 0)
		throw RuntimeError() << this << ":" << src_strerror (error);

	ratio = r;
}

/* /////////////////////////////////////////////////////////////////////// */

static PyObject *
Resampler_resample (Resampler * r, AudioSignal * s)
Py_TRY
{
	if (!AudioSignal_type->is_derived (s))
		throw TypeError() << "needs an AudioSignal.";
	
	int n = (int) (s->size * r->ratio + .5);
	
	AudioSignal * t = NEW (AudioSignal);
	t->set_size (n);
	
	SRC_DATA d = 
	{
		s->data, t->data,

		s->size, t->size,
		0, 0,
		0, /* potentially more data following. */
		r->ratio
	};

	if (src_process (r->src, &d))
	{
		::decref (t, true);
		throw RuntimeError() << r << ":" << src_strerror (r->error);
	}

	return t;
}
Py_CATCH (0);

static PyObject *
Resampler_reset (Resampler * r)
{
	src_reset (r->src);
	return PyNone();
}

static PyMethodDef
Resampler_methods[] = 
{
	{"resample", (PyCFunction) Resampler_resample, METH_O,
	 "resample (audio signal) -- return a resampled copy of the signal."},
	{"reset", (PyCFunction) Resampler_reset, METH_NOARGS,
	 "reset() -- call this when applying the resampler to a different "
	 "signal stream."},
	{ }
};

/* /////////////////////////////////////////////////////////////////////// */

void
Py<Resampler>::type_init (Type * t, PyObject * dict)
{
	t->tp_flags |= Py_TPFLAGS_BASETYPE;
	t->tp_base = Object_type;
	t->tp_methods = Resampler_methods;
}

char * Py<Resampler>::__doc__ = 
"Resampler (ratio, type = 0) -- resamples audio signals.";

void
Py<Resampler>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {"ratio", "type", 0};
	double r;
	int t = 0;
	
	if (!PyArg_ParseTupleAndKeywords (args, kwds, "d|i", keys, &r, &t))
		return;
	
	init (r, t);
}

void
init_Resampler_types (PyObject * dict)
{
	Resampler_type = new PyType<Resampler> ("Resampler", dict);
}

#else /* undefined (WITH_LIBSAMPLERATE) */

void
init_Resampler_types (PyObject * dict)
{
}

#endif /* WITH_LIBSAMPLERATE */
