/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _SCHEDULER_H_
#define _SCHEDULER_H_

extern Type * Scheduler_type;

/* schedulers can be triggered from any cycle. thus, to pass the executing
 * thread a plugin, we need a fifo that is safe to write to from many threads.
 * we resort to using a Pipe, which is a wrapper for pipe(2).
 * 
 * processing requests are queued up in the pipe until a trigger is issued.
 */
class Scheduler
: public Plugin,
	public Thread
{
	private:
		Poll poll;
		Pipe plugin_pipe;
		Pipe trigger_pipe;

		/* Plugin::Script || Plugin::Disk */
		int cycle;
		
		atomic need_trigger;

	public:
		Scheduler (Type * t = Scheduler_type, int _cycle = 0);
		~Scheduler();

		/* this writes a trigger to wake up the thread. */
		virtual void wake_up();

		/* these write the plugin address to the plugin pipe. */
		virtual void process_disk (Plugin * p);
		virtual void process_script (Plugin * p);

	public:
		virtual void * thread();
};

class DiskScheduler
: public Scheduler
{
	public:
		DiskScheduler()
			: Scheduler (DiskScheduler_type, Plugin::Disk)
			{ }
};

class ScriptScheduler
: public Scheduler
{
	public:
		ScriptScheduler()
			: Scheduler (ScriptScheduler_type, Plugin::Script)
			{ }
};

#endif /* _SCHEDULER_H_ */
