/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"

#include "Graph.h"
#include "PyType.h"

#include "dsp/Sine.h"

static Type * Sine_type;

class Sine
: public Plugin
{
	protected:
		DSP::Sine sine;

	public:
		d_float f, next_f;

		Sine();

		void init (d_float sample_rate);

		virtual void process_audio (int frames);
};

/* /////////////////////////////////////////////////////////////////////// */

Sine::Sine()
: Plugin (Sine_type, Audio)
{
	f = next_f = 0;

	add_out (new Out (this, CyclicAudio_type, "out"));
}

void
Sine::init (d_float sample_rate)
{
	f = next_f;

	sine.set_f (f, sample_rate, 0);
}

void 
Sine::process_audio (int frames)
{
	if (f != next_f)
		init (graph->sample_rate);

	CyclicAudio * e = (CyclicAudio *) outs[0]->cache->pop();
	
	e->frame = graph->frame;

	d_sample * s = e->data;

	for (int i = 0; i < frames; ++i)
		s[i] = sine.get();

	outs[0]->push (e);
}

/* /////////////////////////////////////////////////////////////////////// */

static PyObject *
Sine_get_f (Sine * s)
{
	return PyFloat (s->f);
}

static int
Sine_set_f (Sine * s, PyObject * value)
{
	if (!PyArg_Parse (value, "d", &s->next_f))
		return -1;
	
	return 0;
}

static PyGetSetDef 
Sine_getsets [] =
{
	{"f", (getter) Sine_get_f, (setter) Sine_set_f, 
		"frequency."},

	{ }
};

/* /////////////////////////////////////////////////////////////////////// */

void
Py<Sine>::type_init (Type * t, PyObject * dict)
{
	t->tp_flags |= Py_TPFLAGS_BASETYPE;
	t->tp_base = Plugin_type;
	t->tp_getset = Sine_getsets;
}

char * Py<Sine>::__doc__ = 
"Sine (f) -- ";

void
Py<Sine>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {"f", 0};

	if (!PyArg_ParseTupleAndKeywords (args, kwds, "d", keys,
				&next_f))
		return;
}

/* /////////////////////////////////////////////////////////////////////// */

void
init_Sine_types (PyObject * dict)
{
	Sine_type = new PyType<Sine> ("Sine", dict);
}
