/* (c) 2001-2004 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _WAVE_FILE_H_
#define _WAVE_FILE_H_

#include "BinaryFile.h"
#include "SoundFile.h"

extern Type * WaveFile_type;

class WaveFile
: public SoundFile
{
	private:
		int format; /* as per wave spec, only 1 + 3 are supported here */
		int bytes_per_sample;
		int frame_size;
		off_t data; /* sample data offset into file */

		/* 'file pointer' */
		d_frame at;
		
		/* address and length of mmapped file */
		void * map; 
		size_t map_length; 

		BinaryFile file;

	public:
		WaveFile();
		~WaveFile();

		/* bits == 0 means 32 bit IEEE floats; 8, 16 and 32 mean integer
		 * bit depth. */
		void open (char * path, int mode, int channels = 1, 
				int sr = SAMPLE_RATE, int bits = 0);
		
		/* SoundFile:: */
		virtual char * get_path()
			{ 
				return file.path;
			}

		void read (d_sample **, int frames);
		void write (d_sample **, int frames);

		void seek (d_frame);
		d_frame tell();
		
	private:
		/* frame to file offset conversion. */
		inline off_t frame2off (d_frame f)
			{
				return data + f * frame_size;
			}

		void * map_frames (int frames, bool touch = 0);
		void unmap_frames();

		template <class T>
		void read_format_1 (d_sample **, int frames, d_sample factor, d_sample add = 0.0);
		void read_format_3 (d_sample **, int frames);
		
		template <class T>
		void write_format_1 (d_sample **, int frames, d_sample factor, d_sample sub = 0.0);
		void write_format_3 (d_sample **, int frames);
		
		void read_header();
		void write_header();
};

#endif /* !_WAVE_FILE_H_ */
