/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _BASICS_H_
#define _BASICS_H_

#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <values.h>
#include <pthread.h>
#include <exception>
#include <errno.h>
#include <math.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <time.h>

#include "defines.h"

#define NOTE								60
#define VELOCITY						100

#define QUARTER							192
#define BPM									96.0

#define FRAMES_PER_CYCLE		2048
#define SAMPLE_RATE					44100

#define TEN_TO_THE_SIXTH		1000000
#define TEN_TO_THE_NINTH		1000000000

#ifdef min
#undef min
#endif

#ifdef max
#undef max
#endif

template <class T, class U>	
inline T min (T x, U y)	
	{ 
		return x < y ? x : y; 
	}

template <class T, class U>	
inline T max (T x, U y)	
	{ 
		return x > y ? x : y; 
	}

template <class T>  
inline void swap (T & x, T & y) 
	{
		T h = x;
		x = y;
		y = h;
	}

template <class T>  
inline T sgn (T & x) 
	{
		return x < 0 ? (T) -1 : (T) +1;
	}

typedef __uint8_t			byte;

typedef __int8_t			int8;
typedef __uint8_t			uint8;
typedef __int16_t			int16;
typedef __uint16_t		uint16;
typedef __int32_t			int32;
typedef __uint32_t		uint32;
typedef __int64_t			int64;
typedef __uint64_t		uint64;

typedef __int64_t			d_frame;
#define D_FRAME_MAX		0x7FFFFFFFFFFFFFFF

/* Python 'd' value: */
typedef double				d_time;
typedef double				d_tick;
typedef double				d_float;

/* LADSPA_Data, jack_sample_t, Python 'f' value: */
typedef float					d_sample;

/* ticks are counted as integers in the view widget and elsewhere. */
#define D_TICK_MAX		INT_MAX

/* note is in MIDI scale plus fractional. */
typedef double				d_note;
extern d_float				n2f (d_note);
extern d_note					f2n (d_float);

#ifdef __i386__
	#define TRAP asm ("int $3;")

	/* dodge the denormal number performance hit on x86. you may want to
	 * do the same on ppc, too.
	 */
	inline d_sample renormal (d_sample & s)
		{
			if (((*(unsigned int *) &s) & 0x7f800000) == 0)
				return 0.0;
			return s;
		}
#else
	#define TRAP ;

	inline d_sample renormal (d_sample & s)
		{
			return s;
		}
#endif
		
#if __BYTE_ORDER != __LITTLE_ENDIAN && __BYTE_ORDER != __BIG_ENDIAN
	#error "PDP?"
#endif
		
#if __BYTE_ORDER == __LITTLE_ENDIAN
	template <class T> 
	inline T le2host (T t) 
		{ 
			return t; 
		}

	inline uint16 be2host (uint16 u) 
		{ 
			return (u >> 8) | ((u & 0xFF) << 8); 
		}

	inline uint32 be2host (uint32 u) 
		{ 
			return (u >> 24) | ((u & 0xFF0000) >> 8) 
					| ((u & 0xFF00) << 8) | ((u & 0xFF) << 24); 
		}

	inline int16 be2host (int16 i) 
		{
			return (i >> 8) | ((i & 0xFF) << 8); 
		}

	inline int32 be2host (int32 i) 
		{ 
			return (i >> 24) | ((i & 0xFF0000) >> 8) 
					| ((i & 0xFF00) << 8) | ((i & 0xFF) << 24); 
		}

	template <class T>
	inline T host2le (T t) 
		{ 
			return t; 
		}

	inline uint16 host2be (uint16 u) 
		{ 
			return (u >> 8) | ((u & 0xFF) << 8); 
		}

	inline uint32 host2be (uint32 u) 
		{ 
			return (u >> 24) | ((u & 0xFF0000) >> 8) 
					| ((u & 0xFF00) << 8) | ((u & 0xFF) << 24); 
		}

	inline int16 host2be (int16 i) 
		{
			return (i >> 8) | ((i & 0xFF) << 8); 
		}

	inline int32 host2be (int32 i) 
		{ 
			return (i >> 24) | ((i & 0xFF0000) >> 8) 
					| ((i & 0xFF00) << 8) | ((i & 0xFF) << 24); 
		}

#else /* __BIG_ENDIAN, untested. */

	template <class T> 
	inline T be2host (T t) 
		{ 
			return t; 
		}

	inline uint16 le2host (uint16 u) 
		{ 
			return (u >> 8) | ((u & 0xFF) << 8); 
		}

	inline uint32 le2host (uint32 u) 
		{ 
			return (u >> 24) | ((u & 0xFF0000) >> 8) 
					| ((u & 0xFF00) << 8) | ((u & 0xFF) << 24); 
		}

	inline int16 le2host (int16 i) 
		{
			return (i >> 8) | ((i & 0xFF) << 8); 
		}

	inline int32 le2host (int32 i) 
		{ 
			return (i >> 24) | ((i & 0xFF0000) >> 8) 
					| ((i & 0xFF00) << 8) | ((i & 0xFF) << 24); 
		}

	template <class T>
	inline T host2be (T t) 
		{ 
			return t; 
		}

	inline uint16 host2le (uint16 u) 
		{ 
			return (u >> 8) | ((u & 0xFF) << 8); 
		}

	inline uint32 host2le (uint32 u) 
		{ 
			return (u >> 24) | ((u & 0xFF0000) >> 8) 
					| ((u & 0xFF00) << 8) | ((u & 0xFF) << 24); 
		}

	inline int16 host2le (int16 i) 
		{
			return (i >> 8) | ((i & 0xFF) << 8); 
		}

	inline int32 host2le (int32 i) 
		{ 
			return (i >> 24) | ((i & 0xFF0000) >> 8) 
					| ((i & 0xFF00) << 8) | ((i & 0xFF) << 24); 
		}
#endif

/* gettimeofday, returning a double. ~ microsecond resolution and 
 * mostly monotonic so long as nobody fiddles with the system clock. 
 */
static inline d_time
gettimeofday()
{
	static struct timeval tv;
	static long tv_sec = 0;

  ::gettimeofday (&tv, 0);

	if (tv_sec == 0)
		tv_sec = tv.tv_sec;
	
	return (d_time) (tv.tv_sec - tv_sec) + 
			((d_time) tv.tv_usec) / TEN_TO_THE_SIXTH;
}

/* force memory to be physically mapped. */
static inline void
touch_memory (void * mem, size_t size)
{
	char * m = (char *) mem;
	size_t ps = getpagesize() / 2;

	for (size_t i = 0; i < size; i += ps)
		m[i] = 0;
}

/* more basics. */
#include "atomic.h"

#include "String.h"
#include "Object.h"
#include "Exceptions.h"
#include "Geometry.h"
#include "msg.h"

/* holding a reference while the current scope is valid is handy. */
class PyRef
{
	public:
		PyObject * o;

		PyRef (PyObject * _o) 
			{ 
				if (o == 0) throw blewit();
				o = _o; 
			}

		~PyRef()
			{ 
				::decref (o, true); 
			}

		operator PyObject * ()
			{
				return o;
			}
};

#endif /* _BASICS_H_ */
