/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"

blewit::blewit (int _errno)
{
	py_exc = PyExc_RuntimeError;
	
	if (_errno)
		sprintf (excuse, "[Errno %d] %s ", _errno, strerror (_errno));
	else
		*excuse = 0;
}

blewit::blewit (const char * s)
{
	py_exc = PyExc_RuntimeError;

	if (s == 0) *excuse = 0;
	else strcpy (excuse, s);
}

blewit::blewit (const blewit & b)
{
	*this = b;
}

void
blewit::operator = (const blewit & b)
{
	py_exc = b.py_exc;
	strcpy (excuse, b.excuse);
}

void
blewit::throw_at_python()
{
	if (!PyErr_Occurred())
		PyErr_SetString (py_exc, excuse);
}

/* //////////////////////////////////////////////////////////////////////// */

blewit & 
blewit::operator << (const char * s)
{
	int have = strlen (excuse);
	int get = s ? strlen (s) : 12;

	if (have + get > (int) sizeof (excuse))
		msg(), "ignoring extra excuse ", s;
	else if (s)
		strcpy (excuse + strlen (excuse), s);
	else
		strcpy (excuse + strlen (excuse), "(char *) 0");

	return *this;
}

blewit & 
blewit::operator << (int i)
{
	int have = strlen (excuse);
	
	if (have + 32 > (int) sizeof (excuse))
		msg(), "ignoring extra excuse ", i;
	else
		sprintf (excuse + strlen (excuse), "%d", i);
	
	return *this;
}

blewit & 
blewit::operator << (long long ll)
{
	int have = strlen (excuse);
	
	if (have + 32 > (int) sizeof (excuse))
		msg(), "ignoring extra excuse ", ll;
	else
		sprintf (excuse + strlen (excuse), "%lld", ll);
	
	return *this;
}

blewit & 
blewit::operator << (double d)
{
	int have = strlen (excuse);
	
	if (have + 32 > (int) sizeof (excuse))
		msg(), "ignoring extra excuse ", d;
	else
		sprintf (excuse + strlen (excuse), "%.4f", d);
	
	return *this;
}

blewit & 
blewit::operator << (void * v)
{
	int have = strlen (excuse);
	
	if (have + 32 > (int) sizeof (excuse))
		msg(), "ignoring extra excuse ", v;
	else
		sprintf (excuse + strlen (excuse), "<%p>", v);
	
	return *this;
}

blewit & 
blewit::operator << (Object * o)
{
	if (o == 0) return *this << "(Object *) 0 ";

	int have = strlen (excuse);
	
	const char * tp = get_type_name (o);
	const char * ob = get_name (o);
	if (ob == 0) ob = "(char *) 0";

	int get = strlen (tp) + strlen (ob) + 32;
	
	if (have + get > (int) sizeof (excuse))
		msg(), "ignoring extra excuse, object ", o;
	else
		sprintf (excuse + strlen (excuse), "<'%s' Object '%s' at %p> ", tp, ob, o);
	
	return *this;
}
