#! /usr/bin/env python

# (c) 2001-4 tim goetze <tim@quitte.de> 
# use at will, give credit where due and don't complain.

# like most configure scripts, this is a terrible hack that tries to
# pull together information needed to compile this package. it is a bit
# more to my liking than autoconf bloat.

import sys, re, os, traceback
from distutils.sysconfig import get_config_vars

if ('thread' not in sys.builtin_module_names):
	print
	print 78 * '='
	print """Your Python installation does not support multithreading.
Full Stop."""
	print 78 * '='
	sys.exit (1)

py_vars = get_config_vars()

if '--show-py-vars' in sys.argv:
	for v in py_vars.keys():
		print v, ":", py_vars[v]
	sys.exit (0)

## make variables and management ##############################################

DEFINES_IN	= []
CFLAGS_IN		= ['-Wall']
INCLUDES		= ['-I' + py_vars['INCLUDEPY'], ]
LIBS				= [
								"-rdynamic", 
								"-L.", 
								"-L/usr/local/lib",
								"-L/usr/lib", 		
								"-lpthread",
							]
OBJECTS_IN	= []
SOURCES_IN  = ["$(wildcard *.cc)"]
HEADERS_IN  = ["$(wildcard *.h)", "$(wildcard dsp/*.h)"]

def add_flag (list, flag):
	if flag not in list: list.append (flag)

def new_make_var (flag):
	if not flag: return
	if flag[0] != '-': add_flag (OBJECTS_IN, flag)
	if flag[1] in 'lL': add_flag (LIBS, flag)
	elif flag[1] == 'D': add_flag (DEFINES_IN, flag)
	elif flag[1] == 'I': add_flag (INCLUDES, flag)
	elif flag.startswith ('-Wl'): add_flag (LIBS, flag)
	else: add_flag (CFLAGS_IN, flag)

## helpers ####################################################################

def get_cpu():
	w, r = os.popen2 ('uname -m')
	return r.read().strip()

def print_make_variable (file, name, list): 
	if not len (list):
		return
	file.write (name + " = ")
	comma = ''
	for i in list:
		file.write (comma + i)
		comma = ' \\\n\t'
	file.write ('\n\n')

def make_defines_h():
	file = open ("defines.h", "w")
	print >> file, "/* generated by"
	print >> file, " *", ' '.join (sys.argv)
	print >> file, " */\n"
	print >> file, "#ifndef _CONFIG_H_"
	print >> file, "#define _CONFIG_H_\n"
	for d in DEFINES_IN:
		print >> file, "#define", d[2:] # strip -D
	print >> file, "\n#endif /* _CONFIG_H_ */"

# pkg-config #

def pkg_config (lib, flag):
	w, r = os.popen2 ('pkg-config %s %s' % (lib, flag))
	return r.read().strip().split(' ')

def get_modversion (lib):
	return pkg_config (lib, '--modversion')[0]

def get_version (lib):
	return get_modversion (lib).split ('.')

def add_pkg (lib, flag):
	for string in pkg_config (lib, flag):
		new_make_var (string)

# usage #

def usage():
	print sys.argv[0] + """ [--with-option --without-option ... ]
		
options are:"""
	for key in options:
		option = options[key]
		if option.with: 
			with = "[yes]"
		else: 
			with = "[ no]"
		
		print " " + key.rjust(12), with, option.__doc__


## Option Base Class ##########################################################

class Option:
	def __init__ (self, with):
		self.with = with

	def add_flags (self):
		pass

	def add_package (self, pkg, version, define = None):
		if not self.with:
			return

		print "checking for %s ..." % pkg,
		try:
			v = get_modversion (pkg)
			if v < version:
					raise "have %s, want %s" % (v, version)
			print "yes, %s." % (v)
		except:
			print "no."
			raise
		
		add_pkg (pkg, '--cflags --libs')
		if define:
			new_make_var ('-DWITH_%s' % define)

## Options ####################################################################

class FlatsOption (Option):
	"print flat notes instead of sharps."
	def add_flags (self):
		if not self.with:
			return
		new_make_var ('-DWITH_FLATS')

class StdMsgOption (Option):
	"whether to write log info to ./stdmsg"
	def add_flags (self):
		if not self.with:
			return
		new_make_var ('-DWITH_STDMSG')

class IIWUOption (Option):
	"fluidsynth soundfont renderer"
	URL = "fluidsynth.org"
	def add_flags (self):
		if not self.with:
			return
		self.add_package ('fluidsynth', '1.0.3', 'IIWU')

class JpegOption (Option):
	"libjpeg image reading"
	URL = "www.ijg.org"
	def add_flags (self):
		if not self.with:
			return
		new_make_var ('-DWITH_JPEG')
		new_make_var ('-ljpeg')

class JackOption (Option):
	"the Jack audio connection kit."
	URL = "jackit.sf.net"
	def add_flags (self):
		if not self.with:
			return
		self.add_package ('jack', '0.91.1', 'JACK')

class Mpeg3Option (Option):
	"mpeg file reading."
	URL = "heroinewarrior.com/libmpeg3.php3"
	def __init__ (self, with, path = None):
		Option.__init__ (self, with)
		self.path = path

	def add_flags (self):
		if not self.with:
			return
		print """checking for libmpeg3 ...""",
		if not os.access (self.path, os.F_OK):
			raise self.path + " doesn't exist."
		new_make_var ("-I" + self.path)

		libmpeg = self.path + "/" + get_cpu() + "/libmpeg3.a"
		if not os.access (libmpeg, os.R_OK):
			raise libmpeg + " doesn't exist."
		print "yes."
		add_flag (LIBS, libmpeg)
		new_make_var ('-DWITH_MPEG3')

class DebugOption (Option):
	"build with debug information, optimizing turned off."
	def add_flags (self):
		if self.with:
			new_make_var ('-DDEBUG')
			new_make_var ('-g3')
		else:
			new_make_var ('-DNDEBUG')
			new_make_var ('-O2')

class AlsaOption (Option):
	"ALSA"
	URL = "alsa-project.org"
	def add_flags (self):
		if not self.with:
			raise "cannot compile without ALSA."
		self.add_package ('alsa', '1.0.0')

class samplerateOption (Option):
	"samplerate conversion"
	URL = "www.mega-nerd.com/SRC"
	def add_flags (self):
		if not self.with:
			return
		self.add_package ('samplerate', '0.0.15', "SAMPLERATE")

class OggOption (Option):
	"ogg vorbis file reading"
	URL = "xiph.org"
	def add_flags (self):
		if not self.with:
			return
		print "including ogg vorbis support with NO CHECKS made."
		new_make_var ("-DWITH_OGG")
		new_make_var ("-lvorbisfile")
		new_make_var ("-lvorbis")

class FFTWOption (Option):
	"FFTW version 2"
	URL = "fftw.org"
	def add_flags (self):
		if not self.with:
			return
		print "including FFTW 2 support with NO CHECKS made."
		new_make_var ("-DWITH_FFTW")
		new_make_var ("-lsrfftw")
		new_make_var ("-lsfftw")

class GtkOption (Option):
	"windowing with Gtk+ and PyGtk."
	URL = "www.gtk.org"
	PACKAGES = ('gtk+-2.0', 'pygtk-2.0', 'gthread-2.0')

	def add_flags (self):
		if not self.with:
			return
		for p in self.PACKAGES:
			self.add_package (p, '2.0.0')
		new_make_var ('-DWITH_GTK')
		SOURCES_IN.append ("$(wildcard gtk/*.cc)")
		HEADERS_IN.append ("$(wildcard gtk/*.h)")

class RTCOption (Option):
	"use /dev/rtc, the realtime clock."
	URL = "kernel.org"
	def add_flags (self):
		print """checking for the /dev/rtc device file ...""",
		try:
			os.stat ('/dev/rtc')
			print "yes."
		except:
			print "no."
			raise 

		print """checking for the /dev/rtc header file ...""",

		try:
			os.stat ('/usr/include/linux/rtc.h')
			new_make_var ('-DHAVE_RTC_H')
			print "yes."
		except:
			try:
				# versions before 2.4 sometimes have this instead it seems.
				os.stat ('/usr/include/linux/mc146818rtc.h')
				new_make_var ('-DHAVE_MC146818RTC_H')
				print "yes, old style."
			except:
				print "no."
				raise 

## list of options ############################################################

options = {
	'alsa': AlsaOption (1),
	'rtc': RTCOption (1),
	'fluid': IIWUOption (1),
	'jack': JackOption (1),
	'stdmsg': StdMsgOption (1),
	'debug': DebugOption (0),
	'flats': FlatsOption (1),
	'mpeg3': Mpeg3Option (0),
	'gtk': GtkOption (1),
	'samplerate': samplerateOption (1),
	'ogg': OggOption (0),
	'fftw': FFTWOption (0),
	'jpeg': JpegOption (0),
}

## main #######################################################################

if '--help' in sys.argv or '-h' in sys.argv:
	usage()
	sys.exit(0)

cpu = get_cpu()
if not re.compile('^i[3-9]86$').match (cpu):
	print 78 * '='
	print "Please note that you may need to edit the files 'basics.h' and 'atomic.h'"
	print "to match this system's architecture."
	print
	print "Send mail to <tim@quitte.de> if you don't know how to do that, otherwise"
	print "please don't forget to send in your patches."
	print 78 * '='
	print

if not py_vars['MACHDEP'].startswith ('linux'):
	print 78 * '='
	print "You're not compiling on Linux. Expect a rough ride."
	print 78 * '='
	print

# gather libraries necessary to link against python #

for s in py_vars['LIBS'].split():
	new_make_var (s)

# parse options #

args = list (sys.argv[1:])

for name in options:
	option = options[name]
	exp = re.compile ('^--with(out)?-' + name + '(=.*)?')
	
	for arg in args:
		match = exp.match (arg)
		if not match:
			continue
		args.remove (arg)
		
		groups = match.groups()
		if groups[0] == 'out':
			option.with = 0
		elif groups[1]:
			option.with = 1
			option.path = os.path.expanduser (groups[1][1:])
		else:
			option.with = 1

if args: # any args left we couldn't parse? not good.
	print "unknown option(s):", ' '.join (args)
	sys.exit (1)

py_dest = py_vars ['LIBDEST'] + '/site-packages'
if py_dest not in sys.path:
	py_dest = py_vars ['LIBDEST']

PY_DEST = [py_dest]

for name in options:
	option = options[name]
	try: 
		option.add_flags()
	except: 
		print 78 * "="
		print "there was a problem with the build option '%s'." % name
		if hasattr (option, 'URL'):
			print "if you dont have the most recent package, fetch it from"
			print "http://%s" % option.URL
		print 78 * "="
		raise

## make defines.make ##########################################################

print
print "writing 'defines.make'."

file = open ('defines.make', 'w')

print >> file
print >> file, "CONFIGURE_PY_OPTIONS =", ' '.join (sys.argv[1:])
print >> file

for var in ('CFLAGS_IN', 'INCLUDES', 'LIBS', 'PY_DEST', 'OBJECTS_IN', 
		'SOURCES_IN', 'HEADERS_IN'):
	print_make_variable (file, var, eval (var))
		
print "writing 'defines.h'"
make_defines_h()

if options['stdmsg'].with:
	print 
	print 78 * '='
	print "Please read the documentation about the use of the 'stdmsg' file before"
	print "running this software."
	print 78 * '='

print
print "you can invoke 'make' now, but do 'make clean' first if you reconfigured"
print "this package."
print
