/* Copyright 2001-3 tim goetze <tim@quitte.de> -- see 'COPYING'. */

/* get sample from array at fractional index. */

#ifndef _DSP_PHASE_H_
#define _DSP_PHASE_H_

#include "FPTruncateMode.h"

namespace DSP {
	
/* used by SamplePlayer -> PlayingVoice to maintain fractional sample 
 * indices.
 */
	/*
	 * xm1 = x [inpos - 1];
	 * x0  = x [inpos + 0];
	 * x1  = x [inpos + 1];
	 * x2  = x [inpos + 2];
	 * a = (3 * (x0-x1) - xm1 + x2) / 2;
	 * b = 2*x1 + xm1 - (5*x0 + x2) / 2;
	 * c = (x1 - xm1) / 2;
	 * y [outpos] = (((a * finpos) + b) * finpos + c) * finpos + x0;
	 */
class Phase
{
	public:
		int frame;
		float fract;

		Phase (float d = 0)
			{
				*this = d;
			}
		
		inline d_sample get (d_sample * s)
			{
				return s[frame] * (1 - fract) + s[frame+1] * fract;
			}
		
		inline d_sample cubic_get (d_sample * s)
			{
				register double a = 
						(3 * (s[frame] - s[frame+1]) - s[frame-1] + s[frame+2]) * .5;
				register double b =
						2 * s[frame + 1] + s[frame-1] - (5 * s[frame] + s[frame+2]) * .5;
				register double c = 
						(s[frame+1] - s[frame-1]) * .5;
				
				return s[frame] + (((a * fract) + b) * fract + c) * fract;
			}
		
		inline void operator -= (int i)
			{
				frame -= i;
			}

		/* doesn't work with negative 'p'.
		 *
		 * also (on i386) needs to have the fpu set to truncating mode, 
		 * see SamplePlayer::one_cycle. 
		 */
		inline void operator += (float p)
			{
				fract += p;
				int i;
				fistp (fract, i);
				fract -= i;
				frame += i;
			}

		inline operator double ()
			{
				return (double) frame + fract;
			}

		inline double frames_until (float frame)
			{
				return frame - (double) *this;
			}

		inline void loop (float start, float end)
			{
				*this = start + (float) *this - end;
			}

		inline void operator = (int i)
			{
				frame = i;
				fract = 0;
			}

		inline void operator = (float d)
			{
				frame = (int) d;
				fract = (float) (d - frame);
			}
};

} /* namespace DSP */

#endif /* _DSP_PHASE_H_ */
