#ifndef _SSE_4_POLE_H_
#define _SSE_4_POLE_H_

namespace DSP {

class SSE4Pole
{
	public:
		/* we need 5 * float[4] values, each 128-bit aligned. to get the alignment
		 * right, reserve some more and assign array pointers dynamically. 
		 * ('aligned' attribute of gcc-2.95.4 doesn't always work). */
		float state [7 * 4];
		float * a, * b, * x, * y, * t;

		float a0, gain;

		SSE4Pole (float * A, float * B, float g = 1)
		{
			/* take care of 128-bit alignment */
			int s = (int) (char *) state;
			s &= 0xF;
			if (s)
				s = 16 - s;
			
			a = (float *) (((char *) state) + s);
			x = a + 8;
			
			b = a + 4;
			y = a + 16;

			t = a + 20;

			a0 = A[0];
			gain = g;

			for (int i = 0; i < 4; ++i)
				a[i] = A[i + 1],
				b[i] = B[i + 1];
			reset();
		}

		void reset()
		{
			x[0] = y[0] =
			x[1] = y[1] =
			x[2] = y[2] =
			x[3] = y[3] = 0;
		}

		float process (float s)
		{
			/* register usage:
			 *  xmm0: x[]
			 *  xmm1: a[], then a[] * x[], then a[] * x[] + b[] * y[]
			 *  xmm2: y[]
			 *  xmm3: b[], then b[] * y[]
			 */
			
			/* x[i] * a[i], result in xmm1 */
			asm ("movaps %0, %%xmm0" : : "m" (x[0]));
			asm ("movaps %0, %%xmm1" : : "m" (a[0]));
			asm ("mulps %xmm0, %xmm1" : : );
			
			/* y[i] * b[i], result in xmm3 */
			asm ("movaps %0, %%xmm2" : : "m" (y[0]));
			asm ("movaps %0, %%xmm3" : : "m" (b[0]));
			asm ("mulps %xmm2, %xmm3" : : );
			
			/* x[i] * a[i] + y[i] * b[i], result in xmm1 */
			asm ("addps %xmm3, %xmm1" : : );

			/* move the sum vector into t[] -- we do the summing in normal floats */
			asm ("movaps %%xmm1, %0" : : "m" (t[0]));
			
			/* shift x[] and move back to memory */
			asm ("shufps $147, %xmm0, %xmm0");
			asm ("movaps %%xmm0, %0" : : "m" (x[0]));

			/* shift y[] and move back to memory */
			asm ("shufps $147, %xmm2, %xmm2");
			asm ("movaps %%xmm2, %0" : : "m" (y[0]));

			x[0] = s;
			
			s *= a0;
			
			s += t[0];
			s += t[1];
			s += t[2];
			s += t[3];

			y[0] = s;
			
			return s * gain;
		}
};

} /* namespace DSP */

#endif /* _SSE_4_POLE_H_ */
