/*
  dsp/Zoelzer.h

	Copyright 2004 Tim Goetze <tim@quitte.de>

	Udo Zoelzer's biquad prototypes, according to the transcription of
	Chris Townsend.

*/
/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
	02111-1307, USA or point your web browser to http://www.gnu.org.
*/

#ifndef _DSP_ZOELZER_H_
#define _DSP_ZOELZER_H_

#include "util.h"

namespace DSP { namespace Zoelzer {

/* base class, implements the basic parameter calculation */
class Zoelzer
{
	public:
		double K, K2, KoverQ;
		double a[3], b[3];

	public:
		Zoelzer (double f, double Q)
			{
				msg(), f, Q;
				double w = 2 * M_PI * f;

				K = tan (.5 * w);
				K2 = K * K;
				KoverQ = K / Q;
			}

		/* templated so we can set double and float coefficients from the same
		 * piece of code */
		template <class T>
		void make_direct_I (T * ca, T * cb)
			{
				double a0i = 1 / a[0];
				
				ca[0] = b[0] * a0i;
				ca[1] = b[1] * a0i;
				ca[2] = b[2] * a0i;

				cb[0] = 0;
				cb[1] = -a[1] * a0i;
				cb[2] = -a[2] * a0i;
			}
};

class PeakShelve
: public Zoelzer
{
	public:
		double V;

	public:
		PeakShelve (double f, double Q, double dB)
			: Zoelzer (f, Q)
			{
				V = db2lin (dB);
			}
};

/* now the individual prototypes */

class LP
: public Zoelzer
{
	public:
		template <class T>
		LP (double f, double Q, T * ca, T * cb)
			: Zoelzer (f, Q)
			{
				b[0] = K2;
				b[1] = 2 * K2;
				b[2] = K2;
				
				a[0] = 1 + KoverQ + K2;
				a[1] = 2 * (K2 - 1);
				a[2] = 1 - KoverQ + K2;
				
				make_direct_I (ca, cb);
			}
};

} } /* ~namespace DSP::Zoelzer */

#endif /* _DSP_ZOELZER_H_ */
