#! /usr/bin/env python

import sys
from random import random

sys.path.append ('.')
from Mandala import *

gtk_thread = gtk.Gtk()

def click (*args):
	print args

def configure (v, size):
	print >> stdmsg, v.pixels.width, v.pixels.height, size.width, size.height

def dump_args (*args):
	for x in args:
		print repr (x)
		for p in dir (x):
			print p, getattr (x, p)

def select (self, e):
	print repr (e.selection)
	return 1

class Window (gtk.Window):
	def __init__ (self):
		gtk.Window.__init__ (self)
		self.connect ('delete-event', self.delete)
	
	def delete (self, event):
		self.hide()

w = Window()
vb = gtk.VBox()
w.add (vb)
for i in range (4):
	hb = gtk.HBox()
	vb.add (hb)
	for j in range (4):
		b = gtk.Button ("\n %d x %d \n" % (i, j))
		b.connect ("clicked", click)
		hb.append (b)
		
w.show (1)

class Scope (gtk.CurveView):
	def __init__ (self, size = 1024):
		gtk.CurveView.__init__ (self, size)
		self.events = gtk.gdk.BUTTON_PRESS_MASK
		self.connect ("configure-event", self.configure, late=1)
		self.connect ('button-press-event', self.randomize)

	def configure (self, *args):
		pass
	
	def randomize (self, w, e):
		print "randomize!", w, e
		for i in range (len (self)):
			self[i] = (1. - 2. * random())
		self.redraw()
		
w2 = Window()
scope = Scope()
w2.add (scope)
w2.show (1)

m = gtk.Menu()
for i in range (4):
	mi = gtk.Menu()
	m.add (gtk.MenuItem ("menu %d" % i, mi))
	for j in range (4):
		mi.add (gtk.MenuItem ("point %d" % j))

m.add (gtk.MenuItem())
m.add (gtk.MenuItem ('hello,'))
m.add (gtk.MenuItem ('world!'))
#m.popup()

w3 = gtk.Window()
w3.add (gtk.Table (1, 2))

om = gtk.OptionMenu (m);
w3[0].attach (om, (0, 0), (1, 1))

cl = gtk.CList (('hello', 'world'))
w3[0].attach (cl, (0, 1), (1, 2))
for i in range (10):
	cl.append (('hello', '%d' % i))
w3.show (1)

w4 = gtk.Window()
rgb = gtk.RGBView()
w4.add (rgb)
w4.show(1)

class MV (gtk.MidiView):
	def __init__ (v):
		gtk.MidiView.__init__ (v)

	def inout (v, sign, state):
		# zoom or scroll x depending on modifier keys
		control = state & gtk.gdk.CONTROL_MASK
		shifted = state & gtk.gdk.SHIFT_MASK
		altered = state & gtk.gdk.MOD1_MASK

		if shifted:
			if altered:
				v.zoom_y (sign)
			else:
				v.scroll (0, control and 10 * sign or 50 * sign)
		else:
			if altered:
				v.zoom_x (sign)
			else:
				v.scroll (control and 10 * sign or 50 * sign)
			
	def buttonpress (v, x, button):
		b = button.button
		
		if b == 4:
			v.inout (-1, button.state)
		elif b == 5:
			v.inout (+1, button.state)

	def keypress (v, x, key):
		id = key.keyval

		if id == gtk.gdk.Left:
			v.inout (-1, key.state)
				
		elif id == gtk.gdk.Right:
			v.inout (+1, key.state)
			
		if id == gtk.gdk.Up:
			v.inout (-1, key.state | gtk.gdk.SHIFT_MASK)
				
		elif id == gtk.gdk.Down:
			v.inout (+1, key.state | gtk.gdk.SHIFT_MASK)

		elif id == gtk.gdk.Home:
			v.x0 = -20
			v.y0 = -20
			v.redraw()
			v.blit()

		elif id == ord ('r'):
			v.redraw()
			v.blit()

w6 = gtk.Window (w = 800, h = 330)
mv = MV()
w6.add (mv)
w6.connect ('key-press-event', mv.keypress)
w6.events |= gtk.gdk.BUTTON_PRESS_MASK
w6.connect ('button-press-event', mv.buttonpress)

#mf = MidiFile ('../midithing/p6.mid')
#mf = MidiFile ('../midithing/homecoming.mid')
mf = MidiFile ('../midi/bach/suites/airgstr4.mid')

c = 1
for el in mf.tracks:
	r = (c & 4) and 1
	g = (c & 2) and 1
	b = c & 1
	
	m = (((c ^ 24) & 24) >> 3) * 45 + 63
	el.color = ((r * m) << 16) | ((g * m) << 8) | (b * m)
	#print c, r, g, b, m, "%06x" % el.color
	mv.add (el)
	c += 1

w6.show (1)

# sinc integrator
from math import *

def sinc(x):
	if x < 0.000001: return 1
	x *= pi
	return sin (x) / x	

def cot (x):
	return cos (x) / sin (x)
	
def fsinc(x):
	return atan (pi * x)

def plot (f, over = 1, dx = 0.01):
	n = 512
	w7 = gtk.Window (w = 1000, h = 500)
	cv = gtk.VBox()
	w7.add (cv)
	cv.add (gtk.CurveView(n))
	cv.add (gtk.CurveView(n))

	x = 0.0000000001
	y = 0
	S = 0

	for i in xrange (n * over):
		fx = f(x)
		S += dx * (fx + y) / 2
		cv[0][i / over] = fx
		cv[1][i / over] = S
		x += dx
		y = fx

	print "S =", S
	w7.show (1)
	return w7

class Osc:
	def __init__ (o):
		o.p = -4.03

	def sample (o, x):
		
		p = o.p * pi
		if abs (p) < 0.000001: 
			p = 1
		else: 
			p = sin (p) / p

		o.p += 1.0
		return p

osc = Osc()
w7=plot (osc.sample)
