#! /usr/bin/env python
import sys, os, time
from whrandom import randint, random

# make this example runnable in situ (without installing nam.so) 
sys.path += ['.']
from nam import *
import config

quarter = 192 # MIDI ticks / quarter note

graph = Graph (quarter)

# we need a realtime clock to drive midi, and alsa for audio
graph.add (RTClock())
graph.add (AlsaClock (44100, os.geteuid() and 2048 or 128))

# make an iiwu synth, 4 voices polyphony, play the marimba
sf2 = os.path.dirname (sys.argv[0]) + '/marimba.sf2'
iiwu = IIWU (4, sf2)
graph.add (iiwu, 5.)
# the marimba is channel 0 bank 0 instrument 12
iiwu.cmdline ("select 0 0 0 12")

# ALSA audio on card 0, sub 0
pcm_out = AlsaOut (config.pcm)
# add the output 'after' (10. > 5.) so we can feed it from iiwu
graph.add (pcm_out, 10.) 

# prepare the graph for running, configures the plugins
graph.state = Plugin.Prepared

# audio cabling
iiwu.outs[0].connect (pcm_out.ins[0])
iiwu.outs[1].connect (pcm_out.ins[1])

# set ticks / quarter and tempo
# these actually are the default values, so one would not strictly need
# to set them.
quarter = graph.quarter = 192
graph.store_tempo (96.0, tick = 0)
beat = quarter / 4

# make a list of notes ...
list = EventList (MidiEvent)

def note (n, tick):
	# note on
	if ((tick / beat) % 4) in (2,3) and random() > 0.6:
		n -= 12
	list.add (Note (n, randint (92, 96)), tick)
	
# populate it:
tick = 0 
top = 72 # 'c'
up = 65 # lead voice
intervals = [0,3,2,3]

i = 0
scale = []
while 1:
	n = top
	scale = [0,0,0,0]
	for j in range (4):
		n -= intervals[j]
		scale[j] = n
		note (n, tick + j * beat)
	
	if random() > 0.7:
		up = scale[2]

	m = 12
	next_up = up
	for n in scale:
		if abs (up - n) < m:
			m = abs (up - n)
			next_up = n
	up = next_up

	if (i % 4) == 3:
		note (up + 12, tick)
		if random() > 0.6:
			if intervals[1] > 2:
				top -= 1
				intervals[1] -= 1
		else:
			n = randint (1, 2)
			intervals[n] = (0, 1, 3, random() > 0.7 and 2 or 4, 3) [intervals[n]]
	elif random() > 0.3:
		note (up + 12, tick - beat)
		if random() > 0.4:
			note (top + 12, tick)
			
	i = i + 1
	tick += quarter

	# turnaround at 'g'
	if top == 67:
		break
	
for j in range (4):
	note (67 + j, tick)
	tick += beat

graph.loop = (0, tick)
graph.use_loop = 1

# the list is routed into the graph by a Part ...
part = Part (list)
graph.add (part)

# ... into iiwu's midi in.
part.outs[0].connect (iiwu.ins[0])

# turn on processing
graph.state = Plugin.Activated

# turn on the clocks
graph.state = Plugin.Rolling

while 1:
	try: time.sleep (1.)
	except KeyboardInterrupt: break
