#! /usr/bin/env python
import sys
from random import random, randint

sys.path.append ('.')
from nam import *
import config

quarter = 192

class AlsaGraph (Graph):
	def __init__ (self):
		Graph.__init__ (self, quarter)
		self.add (RTClock())
		self.clock = AlsaClock (44100, 2048)
		self.add (self.clock)
		self.add (ScriptScheduler())
		self.pcm = AlsaOut (config.pcm)
		self.add (self.pcm, 10.0)
		self.state = Plugin.Activated

class Pump (ScriptPlugin):
	def __init__ (self):
		ScriptPlugin.__init__ (self)
		self.notes_out = self.new_out ("notes", MidiEvent)
		self.f_out = self.new_out ("f", FloatEvent)
		self.q_out = self.new_out ("Q", FloatEvent)
		self.beat = quarter / 4
		self.n = 0
		self.vmap = (100, 80, 63, 80, 80, 100, 65, 80)

	def process_beat (self, tick, until):
		while tick < until:
			tick += self.beat

			v = self.vmap [self.n]
			self.n = (self.n + 1) % 8
			
			if random() > .8: v /= 2

			n = self.notes_out.cache.pop()
			n.__init__ (0x90, note = 'Eb4', v = int (v * 1.26), tick = tick)
			self.notes_out.push (n)
			
			n = self.notes_out.cache.pop()
			i = randint (2, 3)
			n.__init__ (0x80, note = 'Eb4', tick = tick + i * self.beat / 4)
			self.notes_out.push (n)
			
			e = self.f_out.cache.pop()
			e.__init__ (random() * 3000 + 440, tick)
			self.f_out.push (e)
			
			e = self.q_out.cache.pop()
			e.__init__ (random() * .6 + .2, tick)
			self.q_out.push (e)
			
class Plate (LadspaPlugin):
	def __init__ (self, size = 6., damping = .01):
		LadspaPlugin.__init__ (self, 'plate_1423.so')
		self.ins[0].value = size
		self.ins[1].value = damping

graph = AlsaGraph()

pump = graph.add (Pump(), 0)
noise = graph.add (Noise(), 1)
adsr = graph.add (ADSR(), 2)
vcf = graph.add (VCF (12000), 3)
plate = graph.add (Plate(), 4)

connect = (
	# event cabling
	(pump, 'f', vcf, 'f'),
	(pump, 'Q', vcf, 'resonance'),
	(pump, 'notes', adsr, 'notes in'),
	# audio 
	(noise, 0, adsr, 0),
	(adsr, 0, vcf, 0), 
	#(vcf, 0, plate, 3), 
	(vcf, 0, plate, -1), 
	(plate, 0, graph.pcm, 0), # left out
	(plate, 1, graph.pcm, 1), # right out
)

for o, n, i, m in connect:
	o = type (n) == int and o.outs[n] or o[n]
	i = type (m) == int and i.ins[m] or i[m]
	o.connect (i)

graph.state = Plugin.Rolling
