#! /usr/bin/env python 
import sys, os, time

# make this example runnable in situ (without installing nam.so) 
sys.path = ['.',] + sys.path
from nam import *
import config

if len (sys.argv) == 2:
	file = sys.argv[1]
	hw = 'hw:1'
else:
	hw = sys.argv[1]
	file = sys.argv[2]

if file[-4:] == '.mp3':
	wave = MpegFile (file)
elif file[-4:] == '.ogg':
	wave = OggFile (file)
else:
	wave = WaveFile (file)

graph = Graph()

# need a disk scheduler to drive wave playback
graph.add (DiskScheduler())

# and the wave file is handled by a disk stream
disk_stream = DiskStream (wave)
graph.add (disk_stream, 5.)

# need an AlsaClock to drive Alsa I/O
graph.add (AlsaClock (wave.sample_rate, 2048, 2))
# play to ALSA audio on card 0
# nb: AlsaIn and ~Out won't open the device and make their i/o ports
# available until the graph they reside in is put into Prepared state.
pcm_out = AlsaOut (config.pcm, relaxed = 1)
# add the output 'after' (10. > 5.) so we can feed it from the disk stream
graph.add (pcm_out, 10.) 

# prepare the graph for running, configures the audio device
graph.state = Plugin.Prepared

# connect every channel
for i in range (wave.channels):
	disk_stream.outs[i].connect (pcm_out.ins[i])

# turn on processing
graph.state = Plugin.Activated
graph.state = Plugin.Rolling

while graph.frame < wave.frames:
	time.sleep (1.)
