#! /usr/bin/env python
import sys, os, time

# make this example runnable in situ (without installing nam.so) 
sys.path = ['.',] + sys.path
from nam import *
import config

graph = Graph()
# sample rate and audio fragment size
graph.add (AlsaClock (44100, os.geteuid() and 2048 or 64))

sf2 = os.path.dirname (sys.argv[0]) + '/marimba.sf2'
# iiwu synth, 12 voices polyphony, play the marimba, reverb on
iiwu = IIWU (4, sf2)
iiwu.gain = 1.0
# the marimba itself is instrument #12
graph.add (iiwu, 5.)

pcm_out = AlsaOut (config.pcm)
# add the output 'after' (10. > 5.) so we can feed it from iiwu
graph.add (pcm_out, 10.) 

# prepare the graph for running, configures the audio device
graph.state = Plugin.Prepared

# audio connections
iiwu.outs[0].connect (pcm_out.ins[0])
iiwu.outs[1].connect (pcm_out.ins[1])

# fetch midi input from ALSA card 1, input 0
midi_in = MidiIn (config.midi_in)
graph.add (midi_in)

class Voice2 (ScriptPlugin):
	def __init__ (self):
		ScriptPlugin.__init__ (self)
		self.input = self.new_in ("in", MidiEvent)
		self.output = self.new_out ("out", MidiEvent)
		# interval to add to notes, base is 'c'
		self.scale = (3, 2, 3, 4, 4, 3, 3, 4, 4, 3, 4, 3)

	def process (self, port, events):
		for e in events:
			# process note on and ~off events
			if e.status not in (0x90, 0x80):
				continue
			self.output.push (e)
			n = e.note + self.scale [e.note % 12]
			e = MidiEvent (e.status | e.channel, n, e.v)
			self.output.push (e)

graph.add (ScriptScheduler())
voice2 = Voice2()
graph.add (voice2)

# midi is routed through Voice2 into IIWU
midi_in.outs[0].connect (voice2.ins[0])
voice2.outs[0].connect (iiwu.ins[0])

# turn on processing
graph.state = Plugin.Prepared
iiwu.cmdline ("select 0 0 0 12")
graph.state = Plugin.Rolling
