/* (c) 2004 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _DATA_VIEW_H_
#define _DATA_VIEW_H_

#include "../dsp/Phase.h"
#include "../dsp/ComplexIIR.h"

class DataViewMark
{
	public:
		double index;
		d_sample amplitude;
		Color color;
};

class DataView
: public RGBView
{
	public:
		TypedObjectList<AudioSignal> signals;
		InstanceList<DataViewMark> marks;
			
		Mutex mutex;
		GdkGC * gc;

		double y_align; /* 0 = top, .5 = center, 1 = bottom */
		int y0; /* zero amplitude */
		int x_offset; 
		double y_offset; 
		double clamped;

		int x_scale; /* draw line every x_scale samples */
		d_sample y_scale[3];

		double amp_per_pixel;
		int pixels_per_frame;

		DataView();
		~DataView();
		
		int x2frame (int x)
			{ return x / pixels_per_frame + x_offset; }
		
		int frame2x (int frame)
			{ return (frame - x_offset) * pixels_per_frame; }

		double sample2y (double sample)
			{ return y0 - (sample) / amp_per_pixel; }

		double y2sample (double y)
			{ return (y - y0) * -amp_per_pixel; }

		void draw_scales();
		void draw_mark (DataViewMark & m);
		void draw_signal (int i, const Rect<int> &);
		void draw_signal_1 (int i, const Rect<int> &);

		/* RGBView:: */
		virtual void configure (const Rect<int>);
		
		void redraw (const Rect<int> &);
		void redraw()
			{
				redraw (Rect<int> (0, 0, pixels->width, pixels->height));
			}

		void complete_redraw();
};

/* /////////////////////////////////////////////////////////////////////// */

class PoleZeroView
: public RGBView
{
	public:
		DSP::PZs pzs;
			
		DSP::complex mark;

		int x0, y0;
		double scale;

		PoleZeroView();
		~PoleZeroView();
		
		double x2real (double x)
			{ return (x - x0) / scale; }
		double real2x (double re)
			{ return re * scale + x0; }

		double y2imag (double y)
			{ return (y - y0) / scale; }
		double imag2y (double im)
			{ return im * scale + y0; }

		void draw_unit_circle();
		void draw_poles_zeros();

		/* RGBView:: */
		virtual void configure (const Rect<int>);
		
		void redraw (const Rect<int> &);
		void redraw()
			{
				redraw (Rect<int> (0, 0, pixels->width, pixels->height));
			}

		void complete_redraw();
};

#endif /* _DATA_VIEW_H_ */
