#include <signal.h>

#include <PyType.h>
#include <PythonThread.h>

#include <pygobject.h>
#include <pygtk/pygtk.h>

#include "Gtk.h"

static Type * PyGObject_type;
static bool gtk_up_and_running = false;

Gtk::Gtk()
: Object (Gtk_type),
	Thread ("Gtk")
{
	if (gtk_up_and_running)
		throw AssertionError() << "already running a Gtk thread.";

	gtk_up_and_running = true;

	context = g_main_context_default();

	if (context == 0)
		throw AssertionError() << "cannot obtain default glib main context";

	g_main_context_ref (context);

	g_main_context_set_poll_func (context, Gtk::poll_func);

	start_thread();
}

Gtk::~Gtk()
{
	gtk_main_quit();
	
	/* unlock and unload :) */
	{
		PythonUnlock unlock;
		join_thread();
	}

	g_main_context_unref (context);

	gtk_up_and_running = false;
}

void *
Gtk::thread()
{
	msg(), "Gtk mode: poll takeover.";

	PythonLock lock;
	gtk_main();

	return 0;
}

gint
Gtk::poll_func (GPollFD * fds, guint n, gint timeout)
{
	int r;
	
	/* release the python lock for the duration of poll() */
	{
		PythonUnlock unlock;
		r = poll ((pollfd *) fds, n, timeout);
	}

	if (PyErr_CheckSignals() == -1)
		PyErr_SetNone (PyExc_KeyboardInterrupt);

	return r;
}

/* //////////////////////////////////////////////////////////////////////// */

Type * Gtk_type;

char * Py<Gtk>::__doc__ = 
"Gtk() -- a thread running the Gtk+ event loop.";

void
Py<Gtk>::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = { 0 };

	if (!PyArg_ParseTupleAndKeywords (args, kwds, "", keys))
		throw blewit();
}

void
Py<Gtk>::type_init (Type * t, PyObject * dict)
{
	t->tp_flags |= Py_TPFLAGS_BASETYPE;
	t->tp_base = Object_type;
}

/* //////////////////////////////////////////////////////////////////////// */

void
pygtk_init()
{
	init_pygobject();
	init_pygtk();

	PyObject * m = PyImport_ImportModule("gobject"); 
	
	if (!m)
		throw ImportError() << "could not import the 'gobject' module";
		
	PyGObject_type = (Type *) PyObject_GetAttrString (m, "GObject");
	
	::decref (m, true);
}

GtkWidget * 
get_widget_from_pygtk (PyObject * arg)
{
	PyGObject * o;
	
	if (!PyArg_Parse (arg, "O!", PyGObject_type, &o))
		throw blewit();
	
	if (!GTK_IS_DRAWING_AREA (o->obj))
		throw TypeError() << "need a GtkDrawingArea";

	/* gtk casts ... ugh. */
	return GTK_WIDGET (o->obj);
}

#ifdef DEBUG
static PyObject *
gtkplus_decref (PyObject * self, PyGObject * o)
Py_TRY
{
	if (!PyGObject_type->is_derived (o->ob_type))
		throw TypeError() << "need a pygtk object.";

	Py_DECREF (o);

	return PyNone();
}
Py_CATCH (0);

static PyMethodDef  
module_methods [] = 
{
	{"decref", (PyCFunction) gtkplus_decref, METH_O,
	 "decref (GObject) -- decrement the refcount. don't use this."},
	{ }
};
#endif /* DEBUG */

/* //////////////////////////////////////////////////////////////////////// */

extern void init_Pixel_types (PyObject * dict);
extern void init_RGB_types (PyObject * dict);
extern void init_MidiView_types (PyObject * dict);
extern void init_DataView_types (PyObject * dict);
extern void init_FreeType_types (PyObject * dict);

void
init_Gtk_types (PyObject * dict)
{
	pygtk_init();

	Gtk_type = new PyType<Gtk> ("Gtk", dict);

	init_Pixel_types (dict);
	init_RGB_types (dict);
	init_MidiView_types (dict);
	init_DataView_types (dict);
	init_FreeType_types (dict);

#ifdef DEBUG
	for (PyMethodDef * md = module_methods; md->ml_name; ++md)
	{
		PyObject * m = PyCFunction_New (md, 0);

		if (m == 0)
		{
			msg(), "cannot create method: ", md->ml_name;
			continue;
		}

		if (PyDict_SetItemString (dict, md->ml_name, m))
			msg(), "cannot register method: ", md->ml_name;
	}
#endif
};

void
fini_Gtk_types()
{
	// ::decref (Gtk::dict, true);
	/* gdk rgb is a memory leak it seems. */
}
