/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#include "basics.h"
#include "Thread.h"

#include "Event.h"

PyObject * _msg::stdmsg;
pthread_t _msg::initial_thread = 0;

/* startup initialization. */

bool 
_msg::init (const char * path)
{
	initial_thread = Thread::self();

	stdmsg = PyFile_FromString (const_cast<char *> (path), "a");
	if (stdmsg == 0)
	{
		/* this should never fail. */
		stdmsg = PyFile_FromFile (stderr, "stdmsg", "a", 0);
		return 1;
	}

	#ifdef NDEBUG
	int fd = fileno (PyFile_AsFile (stdmsg));
	fcntl (fd, F_SETFL, fcntl (fd, F_GETFL) | O_NONBLOCK);
	#endif

	return 1;
}

/* constructors. */

_msg::_msg (char * __file__, int __line__)
{ 
	f = 0;

	if (stdmsg)
		f = PyFile_AsFile (stdmsg);	
		
	if (f == 0) 
		f = stderr;

	_file_ = __file__;
	_line_ = __line__;

	if (Thread::self() != initial_thread)
		fprintf (f, "%5ld ", Thread::self());
	else
		fprintf (f, "      ");
}

_msg::_msg (FILE * _f)
{
	f = _f;
	_file_ = 0;
	_line_ = 0;
}

_msg::~_msg()
{ 
	if (_file_ && _line_ >= 0)
		fprintf (f, " (%s:%d)", _file_, _line_);

	if (f == PyFile_AsFile (stdmsg)) 
		fputc ('\n', f); 
	
	fflush (f); 
}

_msg &
_msg::operator , (FloatEvent * e)
{
	fprintf (f, "FloatEvent (%.3f, tick = %.3f) ", e->value, e->tick); 
	return *this;
}

_msg &
_msg::operator , (MidiEvent * e)
{
	if (e->length == 0)
		throw AssertionError() << e << " has no data.";

	if (e->length == 1)
		fprintf (f, "MidiEvent (%02x, %.3f) ", 
				e->data[0], 
				e->tick); 
	
	else if (e->length == 2)
		fprintf (f, "MidiEvent (%02x, %02x, %.3f) ", 
				e->data[0], 
				e->data[1], 
				e->tick); 

	else if (e->length == 3)
		fprintf (f, "MidiEvent (%02x, %02x, %02x, %.3f) ", 
				e->data[0], 
				e->data[1], 
				e->data[2],
				e->tick); 

	else 
	{
		fprintf (f, "MidiEvent [%d] (%02x, ", e->length, e->data[0]);
		int i = 0;
		int n = min (12, e->length);
		while (++i < n)
			fprintf (f, "%02x, ", e->data[i]);
		if (i < e->length)
			fprintf (f, "...,");
		fprintf (f, "%.3f) ", e->tick);
	}

	return *this;
}
