/* (c) 2001, 2002 tim goetze <tim@quitte.de> -- see the file 'COPYING.gz'. */

#ifndef _MSG_H_
#define _MSG_H_

/* we need these often during debugging. */
class FloatEvent;
class MidiEvent;
class BBT;

#define msg() _msg (__FILE__, __LINE__)

class _msg
{
	public:
		static bool init (const char * path = 0);
		
		/* python file object */
		static PyObject * stdmsg;
		static pthread_t initial_thread;

		FILE * f;
		char * _file_;
		int _line_;

		_msg (char * __file__ = 0, int __line__ = -1); 
		_msg (FILE * f);
	 
		~_msg();
	
		operator FILE * () { 
			return f; 
		}

		/* slows down compile time. */
		#if 0
		template <class T>
		_msg & operator << (T t)
			{
				return *this, t;
			}
		#endif

		_msg & operator , (const char * s) 
			{
				if (!s) s = "(char *) 0";
				fputs (s, f);  return *this;
			}
		
		_msg & operator , (long long ll) 
			{
				fprintf (f, "%lld ", ll); return *this;
			}
		
		_msg & operator , (unsigned long long ll) 
			{
				fprintf (f, "%llu ", ll); return *this;
			}
		
		_msg & operator , (long l) 
			{
				fprintf (f, "%ld ", l); return *this;
			}
		
		_msg & operator , (unsigned long l) 
			{
				fprintf (f, "%lu ", l); return *this;
			}
		
		_msg & operator , (int i) 
			{
				fprintf (f, "%d ", i); return *this;
			}
		
		_msg & operator , (unsigned char b) 
			{
				fprintf (f, "%02x ", b); return *this;
			}
		
		_msg & operator , (unsigned int i) 
			{
				fprintf (f, "%u ", i); return *this;
			}
		
		_msg & operator , (const void * p) 
			{
				fprintf (f, "<%p> ", p); return *this;
			}
		
		_msg & operator , (Object * obj)
			{
				fprintf (f, "<%s at %p> ", get_type_name (obj), obj);
				return *this;
			}

		_msg & operator , (float ff) 
			{
				fprintf (f, "%.4f ", ff); 
				return *this;
			}
		
		_msg & operator , (double d) 
			{
				fprintf (f, "%.5f ", d); 
				return *this;
			}
		
		/* more ... */

		_msg & operator , (const Rect<int> & r) 
			{
				fprintf (f, "[%d %d %dx%d] ", r.x, r.y, r.width, r.height); 
				return *this;
			}

		_msg & operator , (const Rect<double> & r) 
			{
				fprintf (f, "[%f %f %fx%f] ", r.x, r.y, r.width, r.height); 
				return *this;
			}
		
		_msg & operator , (const Vector<int> & r) 
			{
				fprintf (f, "[%d %d] ", r.x, r.y); 
				return *this;
			}
		
		_msg & operator , (const Vector<double> & r) 
			{
				fprintf (f, "[%f %f] ", r.x, r.y); 
				return *this;
			}
	
		/* more ... */

		_msg & operator , (FloatEvent *);
		_msg & operator , (MidiEvent *);

		/* more ... */

		_msg & operator , (BBT &);
};

#endif /* _MSG_H_ */
