#! /usr/bin/env python

import sys

sys.path += ['.',]
import nam as M

bg = '#ffffff'

black = 'black'
orange = '#fe9e37'
gray = '#e3e7de'
darkgray = '#c3c7be'
lightgray = '#f7f7eb'
blue = '#1d40d7'

print """<html>
	<head><title>nam """ + M.__version__ + """</title>
		<style type=text/css>
			code, tt, pre {
				font-family: lucida typewriter, lucidatypewriter, monospace; 
			}
			h1, h2, h3, h4, h5, h6 { 
				font-family: Lucida, avantgarde, sans-serif; 
			}
			a	{
				font-weight: normal;
				text-decoration: none;
			}
			a:visited {
				color: #7f7f00;
			}
			a:link {
				color: """, blue, """;
			}

			a.white {
				color: """, bg, """;
				font-family: Lucida, avantgarde, sans-serif; 
				text-decoration: none;
				font-weight: bold;
			}
			a.bold {
				font-family: Lucida, avantgarde, sans-serif; 
				text-decoration: none;
				font-weight: bold;
			}
			a.bare	{
				font-family: Lucida, avantgarde, sans-serif; 
				text-decoration: none;
			}
			tt.orange {
				color: """, orange, """;
			}
		</style>
	</head>
	<body bgcolor=%s>""" % bg, """
		<table border=0 cellpadding=2 cellspacing=3 width=100%%>
			<tr>
				<td bgcolor=%s""" % darkgray,""">
					<font size=+2 face=Lucida,avantgarde,sans-serif color=%s>nam %s</font>
				</td>""" % (bg, M.__version__), """
			</tr>
		</table>
		<table border=0 cellpadding=2 cellspacing=3 width=100%%>"""

#=============================================================================#

def anchor_list (*args):
	text = ""
	for anchor in args:
		if anchor.startswith ('.'):
			name = anchor[1:]
			text += "&nbsp;&nbsp;"
			cls = 'bare'
		else:
			text += "<br>"
			name = anchor
			cls = 'bold'
		text += """<a class=%s href="#%s">%s</a><br>""" % (cls, name, name)
	return text

#----------------------------------------------------------------------------#

all = {}
gtkplus = {}

method = type (M.Graph.add)
property = type (M.Plugin.y)

class Class:
	def __init__ (self, c, name, dict = all):
		self.name = name
		self.cls = c
		self.base = c.__base__
		self.kids = []
		self.attrs = []
		self.const = []
		self.parent = None
		dict[c] = self
		try: 
			if name == 'Gtk': raise RuntimeError
			c()
			self.instantiatable = 1
		except NotImplementedError: 
			self.instantiatable = None
		except: 
			self.instantiatable = 1
		
	def frame (self, dict):
		"set parent and kids relations."
		if not dict.has_key (self.base):
			print >> sys.stderr, self.cls, self.name
			return
		self.parent = dict[self.base]
		self.parent.kids.append (self)

	def make_attrs (self):
		"make a list of all attributes that are not inherited."
		for name in dir (self.cls):
			if name[:2] == '__':
				continue
			if self.parent and name in dir (self.parent.cls):
				continue
			attr = getattr (self.cls, name)
			if type (attr) == method:
				self.attrs.append ((name, attr))
			elif type (attr) == property:
				self.attrs.append ((name, attr))
			else:
				self.const.append ((name, attr))

	def __str__ (self):
		return self.name
	
	def __cmp__ (self, other):
		if self.name < other.name:
			return -1
		elif other.name < self.name:
			return 1
		return 0

def add_class (name, prefix = '', dict = all):
	if name[0] not in "ABCDEFGHIJKLMNOPQRSTUVWXYZ":
		return
	if name.startswith ('_'):
		return
	if name.startswith ('GraphRequest'):
		return
	if name.startswith ('ModuleR'):
		return
	c = eval ('M.' + prefix + name)
	if hasattr (c, '__base__') and c.__base__ == M.GraphRequest:
		return
	if not hasattr (c, '__base__'):
		return
	Class (c, name, dict)

for name in dir (M):
	add_class (name)

for name in dir (M.gtkplus):
	add_class (name, 'gtkplus.', gtkplus)

def prep_dict (dict):
	for c in dict:
		dict[c].frame (dict)
	for c in dict:
		dict[c].make_attrs()
		dict[c].kids.sort()
		
prep_dict (all)
prep_dict (gtkplus)

names = []
for c in all:
	names.append (all[c].name)
names.sort()

#----------------------------------------------------------------------------#

def download():
	print """
		<tr><td></td>
			<td colspan=2>
			<p style=margin-top:5pt;>
				This program is free software; you can redistribute it and/or modify
				it under the terms of the GNU General Public License as published by
				the Free Software Foundation; either version 2 of the License, or
				(at your option) any later version.
			</p>
			<p>
				This program is distributed in the hope that it will be useful,
				but WITHOUT ANY WARRANTY; without even the implied warranty of
				MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
				<a href=COPYING.gz>GNU General Public License</a> for more details.
			</p>
			<p>
				The source package is here: 
				<a href=dists/nam-%s.tar.gz>nam-%s.tar.gz</a>.""" % (M.__version__, M.__version__), """
			</p>
			<p>
				This is a first, pre-pre-pre-alpha release,
				and some things may not work the way one would expect them to. 
				Bug reports are welcome.
			</p>		

			<a name=Compiling>&nbsp;</a>
			<br>
			<h3>Compiling</h3>
			<p>
				Unpack the source tarball and change to the source
				directory:<br>&nbsp;<br>
				<tt>$ tar xfz nam-%s.tar.gz</tt><br>""" % M.__version__, """
				<tt>$ cd nam-%s</tt><br>""" % M.__version__, """
			</p>

			<p>
				Configure the package:<br>&nbsp;<br>
				<tt>$ ./configure.py --help</tt><br>&nbsp;<br>
				Configuration options are printed. You may want to choose a
				default build first; otherwise add options to the following
				command:<br>&nbsp;<br>
				<tt>$ ./configure.py</tt><br>
			</p>

			<p>
				Please understand that the configure script is not very bright
				and mostly trusts you have the right versions of prerequisite
				packages installed.
				In particular, it may not detect an aged ALSA version.
			</p>
			
			<p>
				Build it:<br>&nbsp;<br>
				<tt>$ make</tt><br>
			</p>

			<a name="First Steps">&nbsp;</a>
			<br>
			<h3>First Steps</h3>
			<p>
				After building the Python extension module, you can test it 
				using the applications from the examples directory (please
				read the <a href=#Bugs>section below</a> first). For
				example,
				<br>&nbsp;<br>
				<tt>$ examples/record.py test.wav</tt>
				<br>&nbsp;<br>
				should record two-channel audio from the first soundcard; 
				<br>&nbsp;<br>
				<tt>$ examples/play.py test.wav</tt>
				<br>&nbsp;<br>
				should play the recording. 
			</p>
			<p>
				Other examples are supplied to show how more meaningful applications
				can be built. Hack away and have fun. 
			</p>
			<p>
				If you haven't done so already, please read the 
				<a href=#Notes>notes section</a>
				below about the 'stdmsg' logfile, low-latency signal
				processing requirements and other points of importance 
				before installing.
			</p>
			<p>
				To make the module available outside the source directory, 
				issue (as root)<br>&nbsp;<br>
				<tt># make install</tt>
			</p>
			&nbsp;<br>
		</td></tr>"""

def empty_row (anchor=None):
	if anchor:
		print """<tr><td>&nbsp;<br><a name="%s">&nbsp;</a></td></tr>""" % anchor
	else:
		print "<tr><td>&nbsp;<br>&nbsp;</td></tr>"

def full_row (title, back = gray, fore = bg, anchor = ""):
	anchor = anchor or title
	print "<tr><td><a name='%s'>&nbsp;</td></tr>" % title
	print "<tr><td colspan=4 bgcolor=%s>" % back
	print "<font size=+2 face=Lucida,avantgarde,sans-serif color=%s>%s</font></td></tr>" % (fore, title)

def format_row():
	print """
		<tr>
			<td><tt>""", 7 * '&nbsp;', """</tt></td>
			<td></td>
			<td><tt>""", 2 * '&nbsp;', """</tt></td>
			<td><tt>""", 2 * '&nbsp;', """</tt></td>
		</tr>"""

def html_row (html, cell = ''):
	try: html = open ("_html/%s" % html).read()
	except: pass
	print """
	<tr>
		<td valign=bottom align=left>
			""", cell, """
		</td>
		<td valign=top colspan=2>
			""", html, """
		</td>
		<td>&nbsp;</td>
	</tr>"""

def recur_index (c, ind = ""):
	i = ind and ind[:-3] + " + " or ""
	print i + "<a class=bare href=#%s>%s</a>" % (str (c), str (c))
	kids = [] + c.kids
	kids.sort()
	if len (kids) > 1:
		for k in kids[:-1]:
			recur_index (k, ind + " | ")
	for k in kids[-1:]:
		recur_index (k, ind + "   ")

def index (names):
	first = '*'
	i = 0
	for name in names:
		if name.startswith (first):
			label = name
		else:
			print 
			first = name[0]
			label = "<b>%s</b>%s" % (first, name[1:])
		print "<a class=bare href=#%s>%s</a>" % (name, label)

def bright_row (left="&nbsp;", middle="&nbsp;", back=lightgray, fore='black'):
	print """
		<tr>
			<td>
				<font size=+1 face=Lucida,avantgarde,sans-serif color=%s>%s</font></td>
			<td bgcolor=%s colspan=3>
				<font size=+1 face=Lucida,avantgarde,sans-serif color=%s><b>%s</b></font></td>
			</td>
		</tr>""" % (orange, left, back, fore, middle)

def recur_parents (c, parents = []):
	if not c:
		return parents
	parents.append (c)
	if c.parent:
		return recur_parents (c.parent, parents)
	return parents

def attr_row (name, attr):
	color = darkgray
	if type (attr) == method:
		color = gray
		name += "()"
	elif type (attr) == property:
		color = lightgray

	print "<tr><td valign=top bgcolor=%s><tt>%s</tt></td>" % (color, name)
	print "<td colspan=3 valign=top bgcolor=%s>" % (color,)
	if type (attr) == int:
		if attr < 0x80: print "<tt>%d</tt>" % attr
		else: print "<tt>0x%X</tt> = %d" % (attr, attr)
	elif attr.__doc__:
		for doc in attr.__doc__.split ('\n'):
			hyphen = doc.find (' --')
			if hyphen == -1:
				print doc
			else:
				print "<tt>%s</tt>%s" % (doc [:hyphen], doc[hyphen:])
			print "<br>"
	else:
		pass
	print "</td></tr>"

def derived_cell (c):
	kids = []
	for kid in c.kids:
		kids.append (kid.name)
	kids.sort()

	print "<td colspan=1 valign=top><br><tt>",
	for kid in kids:
		print "<a href=#%s>%s</a>&nbsp;<br>" % (kid, kid)
	print "<br>&nbsp;</tt></td>"

def doc_cell (doc):
	print """
		<td colspan=2 valign=top>
			<br>
			%s<br>&nbsp;
		</td>""" % (doc)

def collect_parent_attrs (c, s = ""):
	c = c.parent
	if not c: return s
	
	for (name, attr) in c.attrs:
		if s: s += ", "
		s += name
	if c.attrs: s += " (<a href=#%s>%s</a>)" % (c.name, c.name)
	return collect_parent_attrs (c, s)

capital = None
def class_header (c, back):
	global capital
	
	if capital != c.name[0]:
		capital = c.name[0]
		print "<tr><td><font size=+5><br>%s</font></td></tr>" % capital

	print "<tr><td><a name=%s>&nbsp;</a></td></tr>" % c.name
	print """
			<tr>
				<td colspan=3 bgcolor=%s>
					<font size=+2 face=Lucida,avantgarde,sans-serif color=%s>
						%s</font>
				</td>
				<td bgcolor=%s align=center>
					<a class=white href=#Reference>Index</a>
				</td></tr>""" % (back, bg, c.name, back)
	
	parents = recur_parents (c.parent, [])
	n = len (parents)
	n = n == 1 and n or n - 1
	if n > 0:
		print """
					<tr>
						<td colspan=4 bgcolor=%s><tt>""" % lightgray
		for p in parents[:n]:
			comma = p != parents[n - 1] and ", " or ""
			print "<a href=#%s>%s</a>%s" % (p.name, p.name, comma)
		print "</tt></td></tr>"

def reference_header (name, dict, objects):
	print """
			</table>
			<table border=0 cellpadding=2 cellspacing=3 width=100%%>"""
	format_row()
	full_row (name, orange, anchor = name)

	print """<tr><td></td>
				<td><pre style=color:%s;>""" % orange
	for object in objects:
		recur_index (object)
		print
	print """</pre></td>
				<td colspan=2 bgcolor=f7f7eb style=margin-left:4pt;>
					<pre>"""
	names = []
	for t in dict:
		names.append (t.__name__)
	names.sort()
	index (names)
	print "</pre></td></tr>"

def tp_cmp (x, y):
	if x.__name__ < y.__name__:
		return -1
	return 1
	
def reference (prefix, dict):
	orphans = []
	for t in dict:
		if not dict[t].parent:
			orphans.append (dict[t])

	reference_header (prefix + "Reference", dict, orphans)
	names = dict.keys()
	names.sort (tp_cmp)
	for name in names:
		c = dict[name]
		doc = c.cls.__doc__
		hyphen = doc.find ('--')
		syn, doc = doc [:hyphen - 1], doc [hyphen + 3:]
		doc = doc.replace ('\n', "<br>&nbsp;<br>")

		class_header (c, c.instantiatable and orange or darkgray)

		print "<tr>"
		derived_cell (c)
		doc_cell (doc)
		print "</tr>"

		if c.instantiatable:
			print """
				<tr>
					<td colspan=4 bgcolor=%s""" % gray, """>
						<p>
							<tt>%s</tt>
						</p>
					</td>	
				</tr>
				<tr><td>&nbsp;</tr></tr>""" % (syn)

		if c.attrs:
			for (name, attr) in c.attrs:
				attr_row (name, attr)
			print """
				<tr><td>&nbsp;</tr></tr>"""
		
		if c.const:
			for (name, attr) in c.const:
				attr_row (name, attr)
			print """
				<tr><td>&nbsp;</tr></tr>"""
		
		s = collect_parent_attrs (c)
		if s:
			print """
				<tr>
					<td colspan=4 bgcolor=%s><tt>%s</tt></td>
				</tr>
				<tr><td>&nbsp;</tr></tr>""" % (lightgray, s)

		empty_row()

contents = "<p>&nbsp;<br>" + \
		anchor_list (
				"Front Matter",
				"Introduction", 
				".Motivation", 
				".Concept", 
				".Why New", 
				".Why Free", 
				"Installation", 
				".Software", 
				".Hardware", 
				".Porting", 
				".Download", 
				".Compiling", 
				".First Steps", 
				".Bugs", 
				"Implementation", 
				".Processing Cycles",
				".Signal Travel",
				".Time",
				".C++ and Python", 
				".Status", 
				"Notes",
				".stdmsg", 
				".Latency", 
				".Whoami", 
				".System Clock", 
				".Source Code", 
				".Writing Plugins", 
				".Final Word", 
				"Reference",
				"Gtk Reference",
				) + "</p>"


#full_row ("nam " + M.__version__, orange)
format_row()

html_row ("about.html",)
bright_row ("", "Contents", lightgray)
html_row (contents + 2 * "&nbsp;<br>",)
empty_row ("Front Matter")
bright_row ("", "Front Matter", lightgray)
html_row ("frontmatter.html",)

full_row ("Introduction", orange)
html_row ("introduction.html")

full_row ("Installation", orange)
html_row ("installation.html")
empty_row ("Download")
bright_row ("", "Download", orange, bg)
download()
empty_row ("Bugs")
bright_row ("", "Known Bugs", lightgray)
html_row ("bugs.html")

empty_row ("Implementation")
full_row ("Design and Implementation", orange, anchor = None)
html_row ("implementation.html")

full_row ("Notes", orange)
html_row ("notes.html")

empty_row()

reference ("", all)

empty_row()

reference ("Gtk ", gtkplus)

#----------------------------------------------------------------------------#

import socket, time, os

print """
	<tr>
		<td colspan=4 align=center>
			<font color=""" + orange + """>
				This file has been created by %s for '%s@%s', <nobr>%s</nobr>.
			</font>
		</td>
	</tr>
	""" % (sys.argv[0], os.getlogin(), socket.gethostname(), 
		time.strftime ('%A %B %d %Y %Z'))

print "</table>"
print "</body>"

